'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var $coroutine, $debug, $find, $findIndex, $forEach, detach, disable, emergencyShutdownHost, enable, endsWith, extractProperty, format, handleInstallSupplementalPack, installAllPatches, installPatch, installSupplementalPack, listMissingPatches, mapToArray, parseXml, ref, restart, restartAgent, set, start, startsWith, stats, stop;

$debug = require('debug')('xo:api:vm');

$find = require('lodash/find');

$findIndex = require('lodash/findIndex');

$forEach = require('lodash/forEach');

endsWith = require('lodash/endsWith');

startsWith = require('lodash/startsWith');

$coroutine = require('bluebird').coroutine;

format = require('json-rpc-peer').format;

ref = require('../utils'), extractProperty = ref.extractProperty, mapToArray = ref.mapToArray, parseXml = ref.parseXml;

set = function set(arg) {
  var host, nameDescription, nameLabel;
  host = arg.host, nameLabel = arg.name_label, nameDescription = arg.name_description;
  return this.getXapi(host).setHostProperties(host._xapiId, {
    nameLabel: nameLabel,
    nameDescription: nameDescription
  });
};

set.description = 'changes the properties of an host';

set.params = {
  id: {
    type: 'string'
  },
  name_label: {
    type: 'string',
    optional: true
  },
  name_description: {
    type: 'string',
    optional: true
  }
};

set.resolve = {
  host: ['id', 'host', 'administrate']
};

exports.set = set;

restart = function restart(arg) {
  var force, host, ref1;
  host = arg.host, force = (ref1 = arg.force) != null ? ref1 : true;
  return this.getXapi(host).rebootHost(host._xapiId, force);
};

restart.description = 'restart the host';

restart.params = {
  id: {
    type: 'string'
  },
  force: {
    type: 'boolean',
    optional: true
  }
};

restart.resolve = {
  host: ['id', 'host', 'operate']
};

exports.restart = restart;

restartAgent = function restartAgent(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).restartHostAgent(host._xapiId);
};

restartAgent.description = 'restart the Xen agent on the host';

restartAgent.params = {
  id: {
    type: 'string'
  }
};

restartAgent.resolve = {
  host: ['id', 'host', 'administrate']
};

exports.restart_agent = restartAgent;

start = function start(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).powerOnHost(host._xapiId);
};

start.description = 'start the host';

start.params = {
  id: {
    type: 'string'
  }
};

start.resolve = {
  host: ['id', 'host', 'operate']
};

exports.start = start;

stop = function stop(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).shutdownHost(host._xapiId);
};

stop.description = 'stop the host';

stop.params = {
  id: {
    type: 'string'
  }
};

stop.resolve = {
  host: ['id', 'host', 'operate']
};

exports.stop = stop;

detach = function detach(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).ejectHostFromPool(host._xapiId);
};

detach.description = 'eject the host of a pool';

detach.params = {
  id: {
    type: 'string'
  }
};

detach.resolve = {
  host: ['id', 'host', 'administrate']
};

exports.detach = detach;

enable = function enable(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).enableHost(host._xapiId);
};

enable.description = 'enable to create VM on the host';

enable.params = {
  id: {
    type: 'string'
  }
};

enable.resolve = {
  host: ['id', 'host', 'administrate']
};

exports.enable = enable;

disable = function disable(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).disableHost(host._xapiId);
};

disable.description = 'disable to create VM on the hsot';

disable.params = {
  id: {
    type: 'string'
  }
};

disable.resolve = {
  host: ['id', 'host', 'administrate']
};

exports.disable = disable;

listMissingPatches = function listMissingPatches(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).listMissingPoolPatchesOnHost(host._xapiId);
};

listMissingPatches.params = {
  host: {
    type: 'string'
  }
};

listMissingPatches.resolve = {
  host: ['host', 'host', 'view']
};

exports.listMissingPatches = listMissingPatches;

listMissingPatches.description = 'return an array of missing new patches in the host';

installPatch = function installPatch(arg) {
  var host, patchUuid;
  host = arg.host, patchUuid = arg.patch;
  return this.getXapi(host).installPoolPatchOnHost(patchUuid, host._xapiId);
};

installPatch.description = 'install a patch on an host';

installPatch.params = {
  host: {
    type: 'string'
  },
  patch: {
    type: 'string'
  }
};

installPatch.resolve = {
  host: ['host', 'host', 'administrate']
};

exports.installPatch = installPatch;

installAllPatches = function installAllPatches(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).installAllPoolPatchesOnHost(host._xapiId);
};

installAllPatches.description = 'install all the missing patches on a host';

installAllPatches.params = {
  host: {
    type: 'string'
  }
};

installAllPatches.resolve = {
  host: ['host', 'host', 'administrate']
};

exports.installAllPatches = installAllPatches;

emergencyShutdownHost = function emergencyShutdownHost(arg) {
  var host;
  host = arg.host;
  return this.getXapi(host).emergencyShutdownHost(host._xapiId);
};

emergencyShutdownHost.description = 'suspend all VMs and shutdown host';

emergencyShutdownHost.params = {
  host: {
    type: 'string'
  }
};

emergencyShutdownHost.resolve = {
  host: ['host', 'host', 'administrate']
};

exports.emergencyShutdownHost = emergencyShutdownHost;

stats = function stats(arg) {
  var granularity, host;
  host = arg.host, granularity = arg.granularity;
  return this.getXapiHostStats(host, granularity);
};

stats.description = 'returns statistic of the host';

stats.params = {
  host: {
    type: 'string'
  },
  granularity: {
    type: 'string',
    optional: true
  }
};

stats.resolve = {
  host: ['host', 'host', 'view']
};

exports.stats = stats;

handleInstallSupplementalPack = $coroutine(_regenerator2.default.mark(function _callee(req, res, arg) {
  var e, hostId, xapi;
  return _regenerator2.default.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          hostId = arg.hostId;
          xapi = this.getXapi(hostId);
          req.setTimeout(43200000);
          req.length = req.headers['content-length'];
          _context.prev = 4;
          _context.next = 7;
          return xapi.installSupplementalPack(req, {
            hostId: hostId
          });

        case 7:
          res.end(format.response(0));
          _context.next = 15;
          break;

        case 10:
          _context.prev = 10;
          _context.t0 = _context['catch'](4);

          e = _context.t0;
          res.writeHead(500);
          res.end(format.error(0, new Error(e.message)));

        case 15:
        case 'end':
          return _context.stop();
      }
    }
  }, _callee, this, [[4, 10]]);
}));

installSupplementalPack = $coroutine(_regenerator2.default.mark(function _callee2(arg) {
  var host;
  return _regenerator2.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          host = arg.host;
          _context2.next = 3;
          return this.registerHttpRequest(handleInstallSupplementalPack, {
            hostId: host.id
          });

        case 3:
          _context2.t0 = _context2.sent;
          return _context2.abrupt('return', {
            $sendTo: _context2.t0
          });

        case 5:
        case 'end':
          return _context2.stop();
      }
    }
  }, _callee2, this);
}));

installSupplementalPack.description = 'installs supplemental pack from ISO file';

installSupplementalPack.params = {
  host: {
    type: 'string'
  }
};

installSupplementalPack.resolve = {
  host: ['host', 'host', 'admin']
};

exports.installSupplementalPack = installSupplementalPack;

Object.defineProperty(exports, '__esModule', {
  value: true
});
//# sourceMappingURL=host.js.map
