'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.delete = exports.delete_ = exports.set = exports.createBonded = exports.create = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var create = exports.create = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(_ref2) {
    var pool = _ref2.pool,
        name = _ref2.name,
        description = _ref2.description,
        pif = _ref2.pif,
        _ref2$mtu = _ref2.mtu,
        mtu = _ref2$mtu === undefined ? 1500 : _ref2$mtu,
        _ref2$vlan = _ref2.vlan,
        vlan = _ref2$vlan === undefined ? 0 : _ref2$vlan;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt('return', this.getXapi(pool).createNetwork({
              name,
              description,
              pifId: pif && this.getObject(pif, 'PIF')._xapiId,
              mtu: +mtu,
              vlan: +vlan
            }));

          case 1:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function create(_x) {
    return _ref.apply(this, arguments);
  };
}();

// =================================================================

var createBonded = exports.createBonded = function () {
  var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(_ref4) {
    var _this = this;

    var pool = _ref4.pool,
        name = _ref4.name,
        description = _ref4.description,
        pifs = _ref4.pifs,
        _ref4$mtu = _ref4.mtu,
        mtu = _ref4$mtu === undefined ? 1500 : _ref4$mtu,
        mac = _ref4.mac,
        bondMode = _ref4.bondMode;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            return _context2.abrupt('return', this.getXapi(pool).createBondedNetwork({
              name,
              description,
              pifIds: (0, _utils.mapToArray)(pifs, function (pif) {
                return _this.getObject(pif, 'PIF')._xapiId;
              }),
              mtu: +mtu,
              mac,
              bondMode
            }));

          case 1:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function createBonded(_x2) {
    return _ref3.apply(this, arguments);
  };
}();

// ===================================================================

var set = exports.set = function () {
  var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(_ref6) {
    var network = _ref6.network,
        nameDescription = _ref6.name_description,
        nameLabel = _ref6.name_label,
        defaultIsLocked = _ref6.defaultIsLocked,
        id = _ref6.id;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return this.getXapi(network).setNetworkProperties(network._xapiId, {
              nameDescription,
              nameLabel,
              defaultIsLocked
            });

          case 2:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function set(_x3) {
    return _ref5.apply(this, arguments);
  };
}();

// =================================================================

var delete_ = exports.delete_ = function () {
  var _ref7 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(_ref8) {
    var network = _ref8.network;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            return _context4.abrupt('return', this.getXapi(network).deleteNetwork(network._xapiId));

          case 1:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function delete_(_x4) {
    return _ref7.apply(this, arguments);
  };
}();

exports.getBondModes = getBondModes;

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getBondModes() {
  return ['balance-slb', 'active-backup', 'lacp'];
}

create.params = {
  pool: { type: 'string' },
  name: { type: 'string' },
  description: { type: 'string', optional: true },
  pif: { type: 'string', optional: true },
  mtu: { type: ['integer', 'string'], optional: true },
  vlan: { type: ['integer', 'string'], optional: true }
};

create.resolve = {
  pool: ['pool', 'pool', 'administrate']
};
create.permission = 'admin';

createBonded.params = {
  pool: { type: 'string' },
  name: { type: 'string' },
  description: { type: 'string', optional: true },
  pifs: {
    type: 'array',
    items: {
      type: 'string'
    }
  },
  mtu: { type: ['integer', 'string'], optional: true },
  // RegExp since schema-inspector does not provide a param check based on an enumeration
  bondMode: { type: 'string', pattern: new RegExp(`^(${getBondModes().join('|')})$`) }
};

createBonded.resolve = {
  pool: ['pool', 'pool', 'administrate']
};
createBonded.permission = 'admin';
createBonded.description = 'Create a bonded network. bondMode can be balance-slb, active-backup or lacp';

set.params = {
  id: {
    type: 'string'
  },
  name_label: {
    type: 'string',
    optional: true
  },
  name_description: {
    type: 'string',
    optional: true
  },
  defaultIsLocked: {
    type: 'boolean',
    optional: true
  }
};

set.resolve = {
  network: ['id', 'network', 'administrate']
};exports.delete = delete_;


delete_.params = {
  id: { type: 'string' }
};

delete_.resolve = {
  network: ['id', 'network', 'administrate']
};
//# sourceMappingURL=network.js.map
