'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAll = exports.delete = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

// -------------------------------------------------------------------

var getAll = exports.getAll = function () {
  var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
    var user;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            user = this.user;

            if (user) {
              _context.next = 3;
              break;
            }

            throw (0, _apiErrors.unauthorized)();

          case 3:
            return _context.abrupt('return', this.getAllResourceSets(user.id));

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getAll() {
    return _ref5.apply(this, arguments);
  };
}();

exports.create = create;
exports.set = set;
exports.get = get;
exports.addObject = addObject;
exports.removeObject = removeObject;
exports.addSubject = addSubject;
exports.removeSubject = removeSubject;
exports.addLimit = addLimit;
exports.removeLimit = removeLimit;
exports.recomputeAllLimits = recomputeAllLimits;

var _apiErrors = require('xo-common/api-errors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ===================================================================

function create(_ref) {
  var name = _ref.name,
      subjects = _ref.subjects,
      objects = _ref.objects,
      limits = _ref.limits;

  return this.createResourceSet(name, subjects, objects, limits);
}

create.permission = 'admin';

create.params = {
  name: {
    type: 'string'
  },
  subjects: {
    type: 'array',
    items: {
      type: 'string'
    },
    optional: true
  },
  objects: {
    type: 'array',
    items: {
      type: 'string'
    },
    optional: true
  },
  limits: {
    type: 'object',
    optional: true
  }
};

// -------------------------------------------------------------------

function delete_(_ref2) {
  var id = _ref2.id;

  return this.deleteResourceSet(id);
}
exports.delete = delete_;


delete_.permission = 'admin';

delete_.params = {
  id: {
    type: 'string'
  }
};

// -------------------------------------------------------------------

function set(_ref3) {
  var id = _ref3.id,
      name = _ref3.name,
      subjects = _ref3.subjects,
      objects = _ref3.objects,
      ipPools = _ref3.ipPools,
      limits = _ref3.limits;

  return this.updateResourceSet(id, {
    limits,
    name,
    objects,
    ipPools,
    subjects
  });
}

set.permission = 'admin';

set.params = {
  id: {
    type: 'string'
  },
  name: {
    type: 'string',
    optional: true
  },
  subjects: {
    type: 'array',
    items: {
      type: 'string'
    },
    optional: true
  },
  objects: {
    type: 'array',
    items: {
      type: 'string'
    },
    optional: true
  },
  ipPools: {
    type: 'array',
    items: {
      type: 'string'
    },
    optional: true
  },
  limits: {
    type: 'object',
    optional: true
  }
};

// -------------------------------------------------------------------

function get(_ref4) {
  var id = _ref4.id;

  return this.getResourceSet(id);
}

get.permission = 'admin';

get.params = {
  id: {
    type: 'string'
  }
};

getAll.description = 'Get the list of all existing resource set';

// -------------------------------------------------------------------

function addObject(_ref6) {
  var id = _ref6.id,
      object = _ref6.object;

  return this.addObjectToResourceSet(object, id);
}

addObject.permission = 'admin';

addObject.params = {
  id: {
    type: 'string'
  },
  object: {
    type: 'string'
  }
};

// -------------------------------------------------------------------

function removeObject(_ref7) {
  var id = _ref7.id,
      object = _ref7.object;

  return this.removeObjectFromResourceSet(object, id);
}

removeObject.permission = 'admin';

removeObject.params = {
  id: {
    type: 'string'
  },
  object: {
    type: 'string'
  }
};

// -------------------------------------------------------------------

function addSubject(_ref8) {
  var id = _ref8.id,
      subject = _ref8.subject;

  return this.addSubjectToResourceSet(subject, id);
}

addSubject.permission = 'admin';

addSubject.params = {
  id: {
    type: 'string'
  },
  subject: {
    type: 'string'
  }
};

// -------------------------------------------------------------------

function removeSubject(_ref9) {
  var id = _ref9.id,
      subject = _ref9.subject;

  return this.removeSubjectFromResourceSet(subject, id);
}

removeSubject.permission = 'admin';

removeSubject.params = {
  id: {
    type: 'string'
  },
  subject: {
    type: 'string'
  }
};

// -------------------------------------------------------------------

function addLimit(_ref10) {
  var id = _ref10.id,
      limitId = _ref10.limitId,
      quantity = _ref10.quantity;

  return this.addLimitToResourceSet(limitId, quantity, id);
}

addLimit.permission = 'admin';

addLimit.params = {
  id: {
    type: 'string'
  },
  limitId: {
    type: 'string'
  },
  quantity: {
    type: 'integer'
  }
};

// -------------------------------------------------------------------

function removeLimit(_ref11) {
  var id = _ref11.id,
      limitId = _ref11.limitId;

  return this.removeLimitFromResourceSet(limitId, id);
}

removeLimit.permission = 'admin';

removeLimit.params = {
  id: {
    type: 'string'
  },
  limitId: {
    type: 'string'
  }
};

// -------------------------------------------------------------------

function recomputeAllLimits() {
  return this.recomputeResourceSetsLimits();
}

recomputeAllLimits.permission = 'admin';
recomputeAllLimits.description = 'Recompute manually the current resource set usage';
//# sourceMappingURL=resource-set.js.map
