'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getVersion = exports.getServerVersion = exports.getServerTimezone = undefined;
exports.getMethodsInfo = getMethodsInfo;
exports.listMethods = listMethods;
exports.methodSignature = methodSignature;

var _forEach = require('lodash/forEach');

var _forEach2 = _interopRequireDefault(_forEach);

var _keys = require('lodash/keys');

var _keys2 = _interopRequireDefault(_keys);

var _momentTimezone = require('moment-timezone');

var _momentTimezone2 = _interopRequireDefault(_momentTimezone);

var _apiErrors = require('xo-common/api-errors');

var _package = require('../../package.json');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ===================================================================

function getMethodsInfo() {
  var methods = {};

  (0, _forEach2.default)(this.apiMethods, function (method, name) {
    methods[name] = {
      description: method.description,
      params: method.params || {},
      permission: method.permission
    };
  });

  return methods;
}
getMethodsInfo.description = 'returns the signatures of all available API methods';

// -------------------------------------------------------------------

var getServerTimezone = exports.getServerTimezone = function (tz) {
  return function () {
    return tz;
  };
}(_momentTimezone2.default.tz.guess());
getServerTimezone.description = 'return the timezone server';

// -------------------------------------------------------------------

var getServerVersion = exports.getServerVersion = function getServerVersion() {
  return _package.version;
};
getServerVersion.description = 'return the version of xo-server';

// -------------------------------------------------------------------

var getVersion = exports.getVersion = function getVersion() {
  return '0.1';
};
getVersion.description = 'API version (unstable)';

// -------------------------------------------------------------------

function listMethods() {
  return (0, _keys2.default)(this.apiMethods);
}
listMethods.description = 'returns the name of all available API methods';

// -------------------------------------------------------------------

function methodSignature(_ref) {
  var name = _ref.method;

  var method = this.apiMethods[name];

  if (!method) {
    throw (0, _apiErrors.noSuchObject)();
  }

  // Return an array for compatibility with XML-RPC.
  return [
  // XML-RPC require the name of the method.
  {
    name,
    description: method.description,
    params: method.params || {},
    permission: method.permission
  }];
}
methodSignature.description = 'returns the signature of an API method';
//# sourceMappingURL=system.js.map
