'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var $assign, $coroutine, $debug, $filter, $findIndex, $findWhere, $isArray, $isVmRunning, $js2xml, attachDisk, attachPci, backup, checkPermissionOnSrs, clone, concat, convertToTemplate, copy, create, createCloudInitConfigDrive, createInterface, delete_, deltaCopy, detachPci, ejectCd, endsWith, escapeStringRegexp, eventToPromise, export_, extract, forEach, forbiddenOperation, format, getCloudInitConfig, handleExport, handleVmImport, importBackup, importDeltaBackup, import_, insertCd, invalidParameters, map, mapFilter, mapToArray, merge, migrate, noop, pCatch, pFinally, parseSize, parseXml, recoveryStart, ref, ref1, restart, resume, revert, rollingBackup, rollingDeltaBackup, rollingDrCopy, rollingSnapshot, set, setBootOrder, snapshot, sortBy, start, startsWith, stats, stop, suspend, unauthorized;

$assign = require('lodash/assign');

$debug = require('debug')('xo:api:vm');

$filter = require('lodash/filter');

$findIndex = require('lodash/findIndex');

$findWhere = require('lodash/find');

concat = require('lodash/concat');

endsWith = require('lodash/endsWith');

escapeStringRegexp = require('escape-string-regexp');

eventToPromise = require('event-to-promise');

merge = require('lodash/merge');

sortBy = require('lodash/sortBy');

startsWith = require('lodash/startsWith');

$coroutine = require('bluebird').coroutine;

format = require('json-rpc-peer').format;

ref = require('xo-common/api-errors'), forbiddenOperation = ref.forbiddenOperation, invalidParameters = ref.invalidParameters, unauthorized = ref.unauthorized;

ref1 = require('../utils'), forEach = ref1.forEach, $js2xml = ref1.formatXml, $isArray = ref1.isArray, map = ref1.map, mapFilter = ref1.mapFilter, mapToArray = ref1.mapToArray, noop = ref1.noop, parseSize = ref1.parseSize, parseXml = ref1.parseXml, pCatch = ref1.pCatch, pFinally = ref1.pFinally;

$isVmRunning = require('../xapi').isVmRunning;

checkPermissionOnSrs = function checkPermissionOnSrs(vm, permission) {
  var permissions;
  if (permission == null) {
    permission = 'operate';
  }
  permissions = [];
  forEach(vm.$VBDs, function (_this) {
    return function (vbdId) {
      var vbd, vdiId;
      vbd = _this.getObject(vbdId, 'VBD');
      vdiId = vbd.VDI;
      if (vbd.is_cd_drive || !vdiId) {
        return;
      }
      return permissions.push([_this.getObject(vdiId, 'VDI').$SR, permission]);
    };
  }(this));
  return this.hasPermissions(this.session.get('user_id'), permissions).then(function (_this) {
    return function (success) {
      if (!success) {
        throw unauthorized();
      }
    };
  }(this));
};

extract = function extract(obj, prop) {
  var value;
  value = obj[prop];
  delete obj[prop];
  return value;
};

create = $coroutine(_regenerator2.default.mark(function _callee2(params) {
  var $resourceSet, checkLimits, existingVdis, i, installation, len, limits, objectIds, ref2, resourceSet, template, user, vdiSizesByDevice, vdis, vif, vifId, vifs, vm, xapi, xapiVm;
  return _regenerator2.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          user = this.user;
          resourceSet = extract(params, 'resourceSet');

          if (!(!resourceSet && user.permission !== 'admin')) {
            _context2.next = 4;
            break;
          }

          throw unauthorized();

        case 4:
          template = extract(params, 'template');
          params.template = template._xapiId;
          xapi = this.getXapi(template);
          objectIds = [template.id];
          limits = {
            cpus: template.CPUs.number,
            disk: 0,
            memory: template.memory.dynamic[1],
            vms: 1
          };
          vdiSizesByDevice = {};
          forEach(xapi.getObject(template._xapiId).$VBDs, function (_this) {
            return function (vbd) {
              var vdi;
              if (vbd.type === 'Disk' && (vdi = vbd.$VDI)) {
                vdiSizesByDevice[vbd.userdevice] = +vdi.virtual_size;
              }
            };
          }(this));
          vdis = extract(params, 'VDIs');
          params.vdis = vdis && map(vdis, function (_this) {
            return function (vdi) {
              var ref2, ref3, size, sr;
              sr = _this.getObject(vdi.SR);
              size = parseSize(vdi.size);
              objectIds.push(sr.id);
              limits.disk += size;
              return $assign({}, vdi, {
                device: (ref2 = (ref3 = vdi.userdevice) != null ? ref3 : vdi.device) != null ? ref2 : vdi.position,
                size: size,
                SR: sr._xapiId,
                type: vdi.type
              });
            };
          }(this));
          existingVdis = extract(params, 'existingDisks');
          params.existingVdis = existingVdis && map(existingVdis, function (_this) {
            return function (vdi, userdevice) {
              var size, sr;
              if (vdi.size != null) {
                size = parseSize(vdi.size);
                vdiSizesByDevice[userdevice] = size;
              }
              if (vdi.$SR) {
                sr = _this.getObject(vdi.$SR);
                objectIds.push(sr.id);
              }
              return $assign({}, vdi, {
                size: size,
                $SR: sr && sr._xapiId
              });
            };
          }(this));
          forEach(vdiSizesByDevice, function (_this) {
            return function (size) {
              return limits.disk += size;
            };
          }(this));
          vifs = extract(params, 'VIFs');
          params.vifs = vifs && map(vifs, function (_this) {
            return function (vif) {
              var network;
              network = _this.getObject(vif.network);
              objectIds.push(network.id);
              return {
                mac: vif.mac,
                network: network._xapiId,
                ipv4_allowed: vif.allowedIpv4Addresses,
                ipv6_allowed: vif.allowedIpv6Addresses
              };
            };
          }(this));
          installation = extract(params, 'installation');
          params.installRepository = installation && installation.repository;
          checkLimits = null;

          if (!resourceSet) {
            _context2.next = 25;
            break;
          }

          _context2.next = 24;
          return this.checkResourceSetConstraints(resourceSet, user.id, objectIds);

        case 24:
          checkLimits = $coroutine(function (_this) {
            return _regenerator2.default.mark(function _callee(limits2) {
              return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      _context.next = 2;
                      return _this.allocateLimitsInResourceSet(limits, resourceSet);

                    case 2:
                      _context.next = 4;
                      return _this.allocateLimitsInResourceSet(limits2, resourceSet);

                    case 4:
                      return _context.abrupt('return', _context.sent);

                    case 5:
                    case 'end':
                      return _context.stop();
                  }
                }
              }, _callee, this);
            });
          }(this));

        case 25:
          _context2.next = 27;
          return xapi.createVm(template._xapiId, params, checkLimits);

        case 27:
          xapiVm = _context2.sent;

          vm = xapi.xo.addObject(xapiVm);

          if (!resourceSet) {
            _context2.next = 44;
            break;
          }

          _context2.t0 = _promise2.default;

          if (!params.share) {
            _context2.next = 38;
            break;
          }

          _context2.next = 34;
          return this.getResourceSet(resourceSet);

        case 34:
          $resourceSet = _context2.sent;
          _context2.t1 = _promise2.default.all(map($resourceSet.subjects, function (_this) {
            return function (subjectId) {
              return _this.addAcl(subjectId, vm.id, 'admin');
            };
          }(this)));
          _context2.next = 39;
          break;

        case 38:
          _context2.t1 = this.addAcl(user.id, vm.id, 'admin');

        case 39:
          _context2.t2 = _context2.t1;
          _context2.t3 = xapi.xo.setData(xapiVm.$id, 'resourceSet', resourceSet);
          _context2.t4 = [_context2.t2, _context2.t3];
          _context2.next = 44;
          return _context2.t0.all.call(_context2.t0, _context2.t4);

        case 44:
          ref2 = vm.VIFs;
          i = 0, len = ref2.length;

        case 46:
          if (!(i < len)) {
            _context2.next = 54;
            break;
          }

          vifId = ref2[i];
          vif = this.getObject(vifId, 'VIF');
          _context2.next = 51;
          return this.allocIpAddresses(vifId, concat(vif.allowedIpv4Addresses, vif.allowedIpv6Addresses))["catch"](function (_this) {
            return function () {
              return xapi.deleteVif(vif._xapiId);
            };
          }(this));

        case 51:
          i++;
          _context2.next = 46;
          break;

        case 54:
          if (params.bootAfterCreate) {
            pCatch.call(xapi.startVm(vm._xapiId), noop);
          }
          return _context2.abrupt('return', vm.id);

        case 56:
        case 'end':
          return _context2.stop();
      }
    }
  }, _callee2, this);
}));

create.params = {
  affinityHost: {
    type: 'string',
    optional: true
  },
  bootAfterCreate: {
    type: 'boolean',
    optional: true
  },
  cloudConfig: {
    type: 'string',
    optional: true
  },
  coreOs: {
    type: 'boolean',
    optional: true
  },
  clone: {
    type: 'boolean',
    optional: true
  },
  resourceSet: {
    type: 'string',
    optional: true
  },
  installation: {
    type: 'object',
    optional: true,
    properties: {
      method: {
        type: 'string'
      },
      repository: {
        type: 'string'
      }
    }
  },
  name_label: {
    type: 'string'
  },
  name_description: {
    type: 'string',
    optional: true
  },
  pv_args: {
    type: 'string',
    optional: true
  },
  share: {
    type: 'boolean',
    optional: true
  },
  template: {
    type: 'string'
  },
  VIFs: {
    optional: true,
    type: 'array',
    items: {
      type: 'object',
      properties: {
        network: {
          type: 'string'
        },
        mac: {
          optional: true,
          type: 'string'
        },
        allowedIpv4Addresses: {
          optional: true,
          type: 'array',
          items: {
            type: 'string'
          }
        },
        allowedIpv6Addresses: {
          optional: true,
          type: 'array',
          items: {
            type: 'string'
          }
        }
      }
    }
  },
  VDIs: {
    optional: true,
    type: 'array',
    items: {
      type: 'object',
      properties: {
        device: {
          type: 'string'
        },
        size: {
          type: ['integer', 'string']
        },
        SR: {
          type: 'string'
        },
        type: {
          type: 'string'
        }
      }
    }
  },
  existingDisks: {
    optional: true,
    type: 'object',
    items: {
      type: 'object',
      properties: {
        size: {
          type: ['integer', 'string'],
          optional: true
        },
        $SR: {
          type: 'string',
          optional: true
        }
      }
    }
  }
};

create.resolve = {
  template: ['template', 'VM-template', '']
};

exports.create = create;

delete_ = $coroutine(_regenerator2.default.mark(function _callee3(arg) {
  var cpus, deleteDisks, disk, ipPoolsUsage, memory, ref2, resourceSet, resourceSetUsage, vdis, vm, xapi;
  return _regenerator2.default.wrap(function _callee3$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          vm = arg.vm, deleteDisks = (ref2 = arg.delete_disks) != null ? ref2 : false;
          cpus = vm.CPUs.number;
          memory = vm.memory.size;
          xapi = this.getXapi(vm);
          this.getAllAcls().then(function (_this) {
            return function (acls) {
              return _promise2.default.all(mapFilter(acls, function (acl) {
                if (acl.object === vm.id) {
                  return pCatch.call(_this.removeAcl(acl.subject, acl.object, acl.action), noop);
                }
              }));
            };
          }(this));
          _context3.next = 7;
          return _promise2.default.all(map(vm.VIFs, function (_this) {
            return function (vifId) {
              var vif;
              vif = xapi.getObject(vifId);
              return pCatch.call(_this.allocIpAddresses(vifId, null, concat(vif.ipv4_allowed, vif.ipv6_allowed)), noop);
            };
          }(this)));

        case 7:
          resourceSet = xapi.xo.getData(vm._xapiId, 'resourceSet');

          if (!(resourceSet != null)) {
            _context3.next = 17;
            break;
          }

          disk = 0;
          vdis = {};
          forEach(vm.$VBDs, function (_this) {
            return function (vbd) {
              var vdi;
              if (vbd.type === 'Disk' && (vdi = vbd.$VDI) && !vdis[vdi.$id]) {
                vdis[vdi.$id] = true;
                disk += +vdi.virtual_size;
              }
            };
          }(this));
          resourceSetUsage = this.computeVmResourcesUsage(vm);
          _context3.next = 15;
          return this.computeVmIpPoolsUsage(vm);

        case 15:
          ipPoolsUsage = _context3.sent;

          pCatch.call(this.releaseLimitsInResourceSet(merge(resourceSetUsage, ipPoolsUsage), resourceSet), noop);

        case 17:
          return _context3.abrupt('return', xapi.deleteVm(vm._xapiId, deleteDisks));

        case 18:
        case 'end':
          return _context3.stop();
      }
    }
  }, _callee3, this);
}));

delete_.params = {
  id: {
    type: 'string'
  },
  delete_disks: {
    optional: true,
    type: 'boolean'
  }
};

delete_.resolve = {
  vm: ['id', ['VM', 'VM-snapshot', 'VM-template'], 'administrate']
};

exports["delete"] = delete_;

ejectCd = $coroutine(_regenerator2.default.mark(function _callee4(arg) {
  var vm;
  return _regenerator2.default.wrap(function _callee4$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          vm = arg.vm;
          _context4.next = 3;
          return this.getXapi(vm).ejectCdFromVm(vm._xapiId);

        case 3:
        case 'end':
          return _context4.stop();
      }
    }
  }, _callee4, this);
}));

ejectCd.params = {
  id: {
    type: 'string'
  }
};

ejectCd.resolve = {
  vm: ['id', 'VM', 'operate']
};

exports.ejectCd = ejectCd;

insertCd = $coroutine(_regenerator2.default.mark(function _callee5(arg) {
  var force, vdi, vm;
  return _regenerator2.default.wrap(function _callee5$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          vm = arg.vm, vdi = arg.vdi, force = arg.force;
          _context5.next = 3;
          return this.getXapi(vm).insertCdIntoVm(vdi._xapiId, vm._xapiId, {
            force: force
          });

        case 3:
        case 'end':
          return _context5.stop();
      }
    }
  }, _callee5, this);
}));

insertCd.params = {
  id: {
    type: 'string'
  },
  cd_id: {
    type: 'string'
  },
  force: {
    type: 'boolean'
  }
};

insertCd.resolve = {
  vm: ['id', 'VM', 'operate'],
  vdi: ['cd_id', 'VDI', 'view']
};

exports.insertCd = insertCd;

migrate = $coroutine(_regenerator2.default.mark(function _callee6(arg) {
  var host, mapVdisSrs, mapVdisSrsXapi, mapVifsNetworks, mapVifsNetworksXapi, migrationNetwork, permissions, vm;
  return _regenerator2.default.wrap(function _callee6$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          vm = arg.vm, host = arg.host, mapVdisSrs = arg.mapVdisSrs, mapVifsNetworks = arg.mapVifsNetworks, migrationNetwork = arg.migrationNetwork;
          permissions = [];
          if (mapVdisSrs) {
            mapVdisSrsXapi = {};
            forEach(mapVdisSrs, function (_this) {
              return function (srId, vdiId) {
                var vdiXapiId;
                vdiXapiId = _this.getObject(vdiId, 'VDI')._xapiId;
                mapVdisSrsXapi[vdiXapiId] = _this.getObject(srId, 'SR')._xapiId;
                return permissions.push([srId, 'administrate']);
              };
            }(this));
          }
          if (mapVifsNetworks) {
            mapVifsNetworksXapi = {};
            forEach(mapVifsNetworks, function (_this) {
              return function (networkId, vifId) {
                var vifXapiId;
                vifXapiId = _this.getObject(vifId, 'VIF')._xapiId;
                mapVifsNetworksXapi[vifXapiId] = _this.getObject(networkId, 'network')._xapiId;
                return permissions.push([networkId, 'administrate']);
              };
            }(this));
          }
          _context6.next = 6;
          return this.hasPermissions(this.session.get('user_id'), permissions);

        case 6:
          if (_context6.sent) {
            _context6.next = 8;
            break;
          }

          throw unauthorized();

        case 8:
          _context6.next = 10;
          return this.getXapi(vm).migrateVm(vm._xapiId, this.getXapi(host), host._xapiId, {
            migrationNetworkId: migrationNetwork != null ? migrationNetwork._xapiId : void 0,
            mapVifsNetworks: mapVifsNetworksXapi,
            mapVdisSrs: mapVdisSrsXapi
          });

        case 10:
        case 'end':
          return _context6.stop();
      }
    }
  }, _callee6, this);
}));

migrate.params = {
  vm: {
    type: 'string'
  },
  targetHost: {
    type: 'string'
  },
  mapVdisSrs: {
    type: 'object',
    optional: true
  },
  mapVifsNetworks: {
    type: 'object',
    optional: true
  },
  migrationNetwork: {
    type: 'string',
    optional: true
  }
};

migrate.resolve = {
  vm: ['vm', 'VM', 'administrate'],
  host: ['targetHost', 'host', 'administrate'],
  migrationNetwork: ['migrationNetwork', 'network', 'administrate']
};

exports.migrate = migrate;

set = function set(params) {
  var VM, xapi;
  VM = extract(params, 'VM');
  xapi = this.getXapi(VM);
  return xapi.editVm(VM._xapiId, params, $coroutine(function (_this) {
    return _regenerator2.default.mark(function _callee7(limits, vm) {
      var error, resourceSet;
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              resourceSet = xapi.xo.getData(vm, 'resourceSet');

              if (!resourceSet) {
                _context7.next = 13;
                break;
              }

              _context7.prev = 2;
              _context7.next = 5;
              return _this.allocateLimitsInResourceSet(limits, resourceSet);

            case 5:
              return _context7.abrupt('return', _context7.sent);

            case 8:
              _context7.prev = 8;
              _context7.t0 = _context7['catch'](2);

              error = _context7.t0;

              if (noSuchObject.is(error)) {
                _context7.next = 13;
                break;
              }

              throw error;

            case 13:
              if (!(limits.cpuWeight && _this.user.permission !== 'admin')) {
                _context7.next = 15;
                break;
              }

              throw unauthorized();

            case 15:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, this, [[2, 8]]);
    });
  }(this)));
};

set.params = {
  id: {
    type: 'string'
  },
  name_label: {
    type: 'string',
    optional: true
  },
  name_description: {
    type: 'string',
    optional: true
  },
  high_availability: {
    type: 'boolean',
    optional: true
  },
  CPUs: {
    type: 'integer',
    optional: true
  },
  cpusMax: {
    type: ['integer', 'string'],
    optional: true
  },
  memory: {
    type: ['integer', 'string'],
    optional: true
  },
  memoryMin: {
    type: ['integer', 'string'],
    optional: true
  },
  memoryMax: {
    type: ['integer', 'string'],
    optional: true
  },
  memoryStaticMax: {
    type: ['integer', 'string'],
    optional: true
  },
  PV_args: {
    type: 'string',
    optional: true
  },
  cpuWeight: {
    type: ['integer', 'null'],
    optional: true
  },
  cpuCap: {
    type: ['integer', 'null'],
    optional: true
  },
  affinityHost: {
    type: ['string', 'null'],
    optional: true
  }
};

set.resolve = {
  VM: ['id', ['VM', 'VM-snapshot', 'VM-template'], 'administrate']
};

exports.set = set;

restart = $coroutine(_regenerator2.default.mark(function _callee8(arg) {
  var force, vm, xapi;
  return _regenerator2.default.wrap(function _callee8$(_context8) {
    while (1) {
      switch (_context8.prev = _context8.next) {
        case 0:
          vm = arg.vm, force = arg.force;
          xapi = this.getXapi(vm);

          if (!force) {
            _context8.next = 7;
            break;
          }

          _context8.next = 5;
          return xapi.call('VM.hard_reboot', vm._xapiRef);

        case 5:
          _context8.next = 9;
          break;

        case 7:
          _context8.next = 9;
          return xapi.call('VM.clean_reboot', vm._xapiRef);

        case 9:
          return _context8.abrupt('return', true);

        case 10:
        case 'end':
          return _context8.stop();
      }
    }
  }, _callee8, this);
}));

restart.params = {
  id: {
    type: 'string'
  },
  force: {
    type: 'boolean'
  }
};

restart.resolve = {
  vm: ['id', 'VM', 'operate']
};

exports.restart = restart;

clone = $coroutine(_regenerator2.default.mark(function _callee9(arg) {
  var full_copy, name, vm;
  return _regenerator2.default.wrap(function _callee9$(_context9) {
    while (1) {
      switch (_context9.prev = _context9.next) {
        case 0:
          vm = arg.vm, name = arg.name, full_copy = arg.full_copy;
          _context9.next = 3;
          return checkPermissionOnSrs.call(this, vm);

        case 3:
          return _context9.abrupt('return', this.getXapi(vm).cloneVm(vm._xapiRef, {
            nameLabel: name,
            fast: !full_copy
          }).then(function (vm) {
            return vm.$id;
          }));

        case 4:
        case 'end':
          return _context9.stop();
      }
    }
  }, _callee9, this);
}));

clone.params = {
  id: {
    type: 'string'
  },
  name: {
    type: 'string'
  },
  full_copy: {
    type: 'boolean'
  }
};

clone.resolve = {
  vm: ['id', 'VM', 'administrate']
};

exports.clone = clone;

copy = $coroutine(_regenerator2.default.mark(function _callee10(arg) {
  var compress, nameLabel, sr, vm;
  return _regenerator2.default.wrap(function _callee10$(_context10) {
    while (1) {
      switch (_context10.prev = _context10.next) {
        case 0:
          compress = arg.compress, nameLabel = arg.name, sr = arg.sr, vm = arg.vm;

          if (!(vm.$pool === sr.$pool)) {
            _context10.next = 6;
            break;
          }

          if (!(vm.power_state === 'Running')) {
            _context10.next = 5;
            break;
          }

          _context10.next = 5;
          return checkPermissionOnSrs.call(this, vm);

        case 5:
          return _context10.abrupt('return', this.getXapi(vm).copyVm(vm._xapiId, sr._xapiId, {
            nameLabel: nameLabel
          }).then(function (vm) {
            return vm.$id;
          }));

        case 6:
          return _context10.abrupt('return', this.getXapi(vm).remoteCopyVm(vm._xapiId, this.getXapi(sr), sr._xapiId, {
            compress: compress,
            nameLabel: nameLabel
          }).then(function (vm) {
            return vm.$id;
          }));

        case 7:
        case 'end':
          return _context10.stop();
      }
    }
  }, _callee10, this);
}));

copy.params = {
  compress: {
    type: 'boolean',
    optional: true
  },
  name: {
    type: 'string',
    optional: true
  },
  vm: {
    type: 'string'
  },
  sr: {
    type: 'string'
  }
};

copy.resolve = {
  vm: ['vm', ['VM', 'VM-snapshot'], 'administrate'],
  sr: ['sr', 'SR', 'operate']
};

exports.copy = copy;

convertToTemplate = $coroutine(_regenerator2.default.mark(function _callee11(arg) {
  var vm;
  return _regenerator2.default.wrap(function _callee11$(_context11) {
    while (1) {
      switch (_context11.prev = _context11.next) {
        case 0:
          vm = arg.vm;
          _context11.next = 3;
          return this.hasPermissions(this.session.get('user_id'), [[vm.$pool, 'administrate']]);

        case 3:
          if (_context11.sent) {
            _context11.next = 5;
            break;
          }

          throw unauthorized();

        case 5:
          _context11.next = 7;
          return this.getXapi(vm).call('VM.set_is_a_template', vm._xapiRef, true);

        case 7:
          return _context11.abrupt('return', true);

        case 8:
        case 'end':
          return _context11.stop();
      }
    }
  }, _callee11, this);
}));

convertToTemplate.params = {
  id: {
    type: 'string'
  }
};

convertToTemplate.resolve = {
  vm: ['id', ['VM', 'VM-snapshot'], 'administrate']
};

exports.convertToTemplate = convertToTemplate;

exports.convert = convertToTemplate;

snapshot = $coroutine(_regenerator2.default.mark(function _callee12(arg) {
  var name, vm;
  return _regenerator2.default.wrap(function _callee12$(_context12) {
    while (1) {
      switch (_context12.prev = _context12.next) {
        case 0:
          vm = arg.vm, name = arg.name;
          _context12.next = 3;
          return checkPermissionOnSrs.call(this, vm);

        case 3:
          _context12.next = 5;
          return this.getXapi(vm).snapshotVm(vm._xapiRef, name != null ? name : vm.name_label + "_" + new Date().toISOString());

        case 5:
          snapshot = _context12.sent;
          return _context12.abrupt('return', snapshot.$id);

        case 7:
        case 'end':
          return _context12.stop();
      }
    }
  }, _callee12, this);
}));

snapshot.params = {
  id: {
    type: 'string'
  },
  name: {
    type: 'string',
    optional: true
  }
};

snapshot.resolve = {
  vm: ['id', 'VM', 'administrate']
};

exports.snapshot = snapshot;

rollingDeltaBackup = $coroutine(_regenerator2.default.mark(function _callee13(arg) {
  var depth, remote, tag, vm;
  return _regenerator2.default.wrap(function _callee13$(_context13) {
    while (1) {
      switch (_context13.prev = _context13.next) {
        case 0:
          vm = arg.vm, remote = arg.remote, tag = arg.tag, depth = arg.depth;
          _context13.next = 3;
          return this.rollingDeltaVmBackup({
            vm: vm,
            remoteId: remote,
            tag: tag,
            depth: depth
          });

        case 3:
          return _context13.abrupt('return', _context13.sent);

        case 4:
        case 'end':
          return _context13.stop();
      }
    }
  }, _callee13, this);
}));

rollingDeltaBackup.params = {
  id: {
    type: 'string'
  },
  remote: {
    type: 'string'
  },
  tag: {
    type: 'string'
  },
  depth: {
    type: ['string', 'number']
  }
};

rollingDeltaBackup.resolve = {
  vm: ['id', ['VM', 'VM-snapshot'], 'administrate']
};

rollingDeltaBackup.permission = 'admin';

exports.rollingDeltaBackup = rollingDeltaBackup;

importDeltaBackup = function importDeltaBackup(arg) {
  var filePath, remote, sr;
  sr = arg.sr, remote = arg.remote, filePath = arg.filePath;
  return this.importDeltaVmBackup({
    sr: sr,
    remoteId: remote,
    filePath: filePath
  });
};

importDeltaBackup.params = {
  sr: {
    type: 'string'
  },
  remote: {
    type: 'string'
  },
  filePath: {
    type: 'string'
  }
};

importDeltaBackup.resolve = {
  sr: ['sr', 'SR', 'operate']
};

importDeltaBackup.permission = 'admin';

exports.importDeltaBackup = importDeltaBackup;

deltaCopy = function deltaCopy(arg) {
  var sr, vm;
  vm = arg.vm, sr = arg.sr;
  return this.deltaCopyVm(vm, sr);
};

deltaCopy.params = {
  id: {
    type: 'string'
  },
  sr: {
    type: 'string'
  }
};

deltaCopy.resolve = {
  vm: ['id', 'VM', 'operate'],
  sr: ['sr', 'SR', 'operate']
};

exports.deltaCopy = deltaCopy;

rollingSnapshot = $coroutine(_regenerator2.default.mark(function _callee14(arg) {
  var depth, tag, vm;
  return _regenerator2.default.wrap(function _callee14$(_context14) {
    while (1) {
      switch (_context14.prev = _context14.next) {
        case 0:
          vm = arg.vm, tag = arg.tag, depth = arg.depth;
          _context14.next = 3;
          return checkPermissionOnSrs.call(this, vm);

        case 3:
          _context14.next = 5;
          return this.rollingSnapshotVm(vm, tag, depth);

        case 5:
          return _context14.abrupt('return', _context14.sent);

        case 6:
        case 'end':
          return _context14.stop();
      }
    }
  }, _callee14, this);
}));

rollingSnapshot.params = {
  id: {
    type: 'string'
  },
  tag: {
    type: 'string'
  },
  depth: {
    type: 'number'
  }
};

rollingSnapshot.resolve = {
  vm: ['id', 'VM', 'administrate']
};

rollingSnapshot.description = 'Snapshots a VM with a tagged name, and removes the oldest snapshot with the same tag according to depth';

exports.rollingSnapshot = rollingSnapshot;

backup = $coroutine(_regenerator2.default.mark(function _callee15(arg) {
  var compress, file, onlyMetadata, remoteId, vm;
  return _regenerator2.default.wrap(function _callee15$(_context15) {
    while (1) {
      switch (_context15.prev = _context15.next) {
        case 0:
          vm = arg.vm, remoteId = arg.remoteId, file = arg.file, compress = arg.compress, onlyMetadata = arg.onlyMetadata;
          _context15.next = 3;
          return this.backupVm({
            vm: vm,
            remoteId: remoteId,
            file: file,
            compress: compress,
            onlyMetadata: onlyMetadata
          });

        case 3:
          return _context15.abrupt('return', _context15.sent);

        case 4:
        case 'end':
          return _context15.stop();
      }
    }
  }, _callee15, this);
}));

backup.permission = 'admin';

backup.params = {
  id: {
    type: 'string'
  },
  remoteId: {
    type: 'string'
  },
  file: {
    type: 'string'
  },
  compress: {
    type: 'boolean',
    optional: true
  },
  onlyMetadata: {
    type: 'boolean',
    optional: true
  }
};

backup.resolve = {
  vm: ['id', 'VM', 'administrate']
};

backup.description = 'Exports a VM to the file system';

exports.backup = backup;

importBackup = function importBackup(arg) {
  var file, remote, sr;
  remote = arg.remote, file = arg.file, sr = arg.sr;
  return this.importVmBackup(remote, file, sr);
};

importBackup.permission = 'admin';

importBackup.description = 'Imports a VM into host, from a file found in the chosen remote';

importBackup.params = {
  remote: {
    type: 'string'
  },
  file: {
    type: 'string'
  },
  sr: {
    type: 'string'
  }
};

importBackup.resolve = {
  sr: ['sr', 'SR', 'operate']
};

importBackup.permission = 'admin';

exports.importBackup = importBackup;

rollingBackup = $coroutine(_regenerator2.default.mark(function _callee16(arg) {
  var compress, depth, onlyMetadata, remoteId, tag, vm;
  return _regenerator2.default.wrap(function _callee16$(_context16) {
    while (1) {
      switch (_context16.prev = _context16.next) {
        case 0:
          vm = arg.vm, remoteId = arg.remoteId, tag = arg.tag, depth = arg.depth, compress = arg.compress, onlyMetadata = arg.onlyMetadata;
          _context16.next = 3;
          return this.rollingBackupVm({
            vm: vm,
            remoteId: remoteId,
            tag: tag,
            depth: depth,
            compress: compress,
            onlyMetadata: onlyMetadata
          });

        case 3:
          return _context16.abrupt('return', _context16.sent);

        case 4:
        case 'end':
          return _context16.stop();
      }
    }
  }, _callee16, this);
}));

rollingBackup.permission = 'admin';

rollingBackup.params = {
  id: {
    type: 'string'
  },
  remoteId: {
    type: 'string'
  },
  tag: {
    type: 'string'
  },
  depth: {
    type: 'number'
  },
  compress: {
    type: 'boolean',
    optional: true
  }
};

rollingBackup.resolve = {
  vm: ['id', ['VM', 'VM-snapshot'], 'administrate']
};

rollingBackup.description = 'Exports a VM to the file system with a tagged name, and removes the oldest backup with the same tag according to depth';

exports.rollingBackup = rollingBackup;

rollingDrCopy = function rollingDrCopy(arg) {
  var depth, pool, sr, tag, vm;
  vm = arg.vm, pool = arg.pool, sr = arg.sr, tag = arg.tag, depth = arg.depth;
  if (!sr) {
    if (!pool) {
      throw invalidParameters('either pool or sr param should be specified');
    }
    if (vm.$pool === pool.id) {
      throw forbiddenOperation('Disaster Recovery attempts to copy on the same pool');
    }
    sr = this.getObject(pool.default_SR, 'SR');
  }
  return this.rollingDrCopyVm({
    vm: vm,
    sr: sr,
    tag: tag,
    depth: depth
  });
};

rollingDrCopy.params = {
  depth: {
    type: 'number'
  },
  id: {
    type: 'string'
  },
  pool: {
    type: 'string',
    optional: true
  },
  sr: {
    type: 'string',
    optional: true
  },
  tag: {
    type: 'string'
  }
};

rollingDrCopy.resolve = {
  vm: ['id', ['VM', 'VM-snapshot'], 'administrate'],
  pool: ['pool', 'pool', 'administrate'],
  sr: ['sr', 'SR', 'administrate']
};

rollingDrCopy.description = 'Copies a VM to a different pool, with a tagged name, and removes the oldest VM with the same tag from this pool, according to depth';

exports.rollingDrCopy = rollingDrCopy;

start = function start(arg) {
  var vm;
  vm = arg.vm;
  return this.getXapi(vm).startVm(vm._xapiId);
};

start.params = {
  id: {
    type: 'string'
  }
};

start.resolve = {
  vm: ['id', 'VM', 'operate']
};

exports.start = start;

stop = $coroutine(_regenerator2.default.mark(function _callee17(arg) {
  var error, force, vm, xapi;
  return _regenerator2.default.wrap(function _callee17$(_context17) {
    while (1) {
      switch (_context17.prev = _context17.next) {
        case 0:
          vm = arg.vm, force = arg.force;
          xapi = this.getXapi(vm);

          if (!force) {
            _context17.next = 6;
            break;
          }

          _context17.next = 5;
          return xapi.call('VM.hard_shutdown', vm._xapiRef);

        case 5:
          return _context17.abrupt('return', true);

        case 6:
          _context17.prev = 6;
          _context17.next = 9;
          return xapi.call('VM.clean_shutdown', vm._xapiRef);

        case 9:
          _context17.next = 19;
          break;

        case 11:
          _context17.prev = 11;
          _context17.t0 = _context17['catch'](6);

          error = _context17.t0;

          if (!(error.code === 'VM_MISSING_PV_DRIVERS' || error.code === 'VM_LACKS_FEATURE_SHUTDOWN')) {
            _context17.next = 18;
            break;
          }

          throw invalidParameters('clean shutdown requires PV drivers');

        case 18:
          throw error;

        case 19:
          return _context17.abrupt('return', true);

        case 20:
        case 'end':
          return _context17.stop();
      }
    }
  }, _callee17, this, [[6, 11]]);
}));

stop.params = {
  id: {
    type: 'string'
  },
  force: {
    type: 'boolean',
    optional: true
  }
};

stop.resolve = {
  vm: ['id', 'VM', 'operate']
};

exports.stop = stop;

suspend = $coroutine(_regenerator2.default.mark(function _callee18(arg) {
  var vm;
  return _regenerator2.default.wrap(function _callee18$(_context18) {
    while (1) {
      switch (_context18.prev = _context18.next) {
        case 0:
          vm = arg.vm;
          _context18.next = 3;
          return this.getXapi(vm).call('VM.suspend', vm._xapiRef);

        case 3:
          return _context18.abrupt('return', true);

        case 4:
        case 'end':
          return _context18.stop();
      }
    }
  }, _callee18, this);
}));

suspend.params = {
  id: {
    type: 'string'
  }
};

suspend.resolve = {
  vm: ['id', 'VM', 'operate']
};

exports.suspend = suspend;

resume = function resume(arg) {
  var vm;
  vm = arg.vm;
  return this.getXapi(vm).resumeVm(vm._xapiId);
};

resume.params = {
  id: {
    type: 'string'
  }
};

resume.resolve = {
  vm: ['id', 'VM', 'operate']
};

exports.resume = resume;

revert = function revert(arg) {
  var snapshot, snapshotBefore;
  snapshot = arg.snapshot, snapshotBefore = arg.snapshotBefore;
  return this.getXapi(snapshot).revertVm(snapshot._xapiId, snapshotBefore);
};

revert.params = {
  id: {
    type: 'string'
  },
  snapshotBefore: {
    type: 'boolean',
    optional: true
  }
};

revert.resolve = {
  snapshot: ['id', 'VM-snapshot', 'administrate']
};

exports.revert = revert;

handleExport = $coroutine(_regenerator2.default.mark(function _callee19(req, res, arg) {
  var compress, id, onlyMetadata, ref2, stream, xapi;
  return _regenerator2.default.wrap(function _callee19$(_context19) {
    while (1) {
      switch (_context19.prev = _context19.next) {
        case 0:
          xapi = arg.xapi, id = arg.id, compress = arg.compress, onlyMetadata = arg.onlyMetadata;
          _context19.next = 3;
          return xapi.exportVm(id, {
            compress: compress != null ? compress : true,
            onlyMetadata: onlyMetadata != null ? onlyMetadata : false
          });

        case 3:
          stream = _context19.sent;

          res.on('close', function () {
            return stream.cancel();
          });
          stream.headers['content-disposition'] = 'attachment';
          res.writeHead(stream.statusCode, (ref2 = stream.statusMessage) != null ? ref2 : '', stream.headers);
          stream.pipe(res);

        case 8:
        case 'end':
          return _context19.stop();
      }
    }
  }, _callee19, this);
}));

export_ = $coroutine(_regenerator2.default.mark(function _callee20(arg) {
  var compress, data, onlyMetadata, vm;
  return _regenerator2.default.wrap(function _callee20$(_context20) {
    while (1) {
      switch (_context20.prev = _context20.next) {
        case 0:
          vm = arg.vm, compress = arg.compress, onlyMetadata = arg.onlyMetadata;

          if (!(vm.power_state === 'Running')) {
            _context20.next = 4;
            break;
          }

          _context20.next = 4;
          return checkPermissionOnSrs.call(this, vm);

        case 4:
          data = {
            xapi: this.getXapi(vm),
            id: vm._xapiId,
            compress: compress,
            onlyMetadata: onlyMetadata
          };
          _context20.next = 7;
          return this.registerHttpRequest(handleExport, data, {
            suffix: encodeURI("/" + vm.name_label + ".xva")
          });

        case 7:
          _context20.t0 = _context20.sent;
          return _context20.abrupt('return', {
            $getFrom: _context20.t0
          });

        case 9:
        case 'end':
          return _context20.stop();
      }
    }
  }, _callee20, this);
}));

export_.params = {
  vm: {
    type: 'string'
  },
  compress: {
    type: 'boolean',
    optional: true
  },
  onlyMetadata: {
    type: 'boolean',
    optional: true
  }
};

export_.resolve = {
  vm: ['vm', ['VM', 'VM-snapshot'], 'administrate']
};

exports["export"] = export_;

handleVmImport = $coroutine(_regenerator2.default.mark(function _callee21(req, res, arg) {
  var data, e, srId, type, vm, xapi;
  return _regenerator2.default.wrap(function _callee21$(_context21) {
    while (1) {
      switch (_context21.prev = _context21.next) {
        case 0:
          data = arg.data, srId = arg.srId, type = arg.type, xapi = arg.xapi;
          req.setTimeout(43200000);
          _context21.prev = 2;
          _context21.next = 5;
          return xapi.importVm(req, {
            data: data,
            srId: srId,
            type: type
          });

        case 5:
          vm = _context21.sent;

          res.end(format.response(0, vm.$id));
          _context21.next = 14;
          break;

        case 9:
          _context21.prev = 9;
          _context21.t0 = _context21['catch'](2);

          e = _context21.t0;
          res.writeHead(500);
          res.end(format.error(0, new Error(e.message)));

        case 14:
        case 'end':
          return _context21.stop();
      }
    }
  }, _callee21, this, [[2, 9]]);
}));

import_ = $coroutine(_regenerator2.default.mark(function _callee22(arg) {
  var data, host, sr, type, xapi;
  return _regenerator2.default.wrap(function _callee22$(_context22) {
    while (1) {
      switch (_context22.prev = _context22.next) {
        case 0:
          data = arg.data, host = arg.host, sr = arg.sr, type = arg.type;

          if (!(data && type === 'xva')) {
            _context22.next = 3;
            break;
          }

          throw invalidParameters('unsupported field data for the file type xva');

        case 3:
          if (sr) {
            _context22.next = 12;
            break;
          }

          if (host) {
            _context22.next = 6;
            break;
          }

          throw invalidParameters('you must provide either host or SR');

        case 6:
          xapi = this.getXapi(host);
          sr = xapi.pool.$default_SR;

          if (sr) {
            _context22.next = 10;
            break;
          }

          throw invalidParameters('there is not default SR in this pool');

        case 10:
          _context22.next = 13;
          break;

        case 12:
          xapi = this.getXapi(sr);

        case 13:
          _context22.next = 15;
          return this.registerHttpRequest(handleVmImport, {
            data: data,
            srId: sr._xapiId,
            type: type,
            xapi: xapi
          });

        case 15:
          _context22.t0 = _context22.sent;
          return _context22.abrupt('return', {
            $sendTo: _context22.t0
          });

        case 17:
        case 'end':
          return _context22.stop();
      }
    }
  }, _callee22, this);
}));

import_.params = {
  data: {
    type: 'object',
    optional: true,
    properties: {
      descriptionLabel: {
        type: 'string'
      },
      disks: {
        type: 'array',
        items: {
          type: 'object',
          properties: {
            capacity: {
              type: 'integer'
            },
            descriptionLabel: {
              type: 'string'
            },
            nameLabel: {
              type: 'string'
            },
            path: {
              type: 'string'
            },
            position: {
              type: 'integer'
            }
          }
        },
        optional: true
      },
      memory: {
        type: 'integer'
      },
      nameLabel: {
        type: 'string'
      },
      nCpus: {
        type: 'integer'
      },
      networks: {
        type: 'array',
        items: {
          type: 'string'
        },
        optional: true
      }
    }
  },
  host: {
    type: 'string',
    optional: true
  },
  type: {
    type: 'string',
    optional: true
  },
  sr: {
    type: 'string',
    optional: true
  }
};

import_.resolve = {
  host: ['host', 'host', 'administrate'],
  sr: ['sr', 'SR', 'administrate']
};

exports["import"] = import_;

attachDisk = $coroutine(_regenerator2.default.mark(function _callee23(arg) {
  var bootable, mode, position, vdi, vm;
  return _regenerator2.default.wrap(function _callee23$(_context23) {
    while (1) {
      switch (_context23.prev = _context23.next) {
        case 0:
          vm = arg.vm, vdi = arg.vdi, position = arg.position, mode = arg.mode, bootable = arg.bootable;
          _context23.next = 3;
          return this.getXapi(vm).attachVdiToVm(vdi._xapiId, vm._xapiId, {
            bootable: bootable,
            position: position,
            readOnly: mode === 'RO'
          });

        case 3:
        case 'end':
          return _context23.stop();
      }
    }
  }, _callee23, this);
}));

attachDisk.params = {
  bootable: {
    type: 'boolean',
    optional: true
  },
  mode: {
    type: 'string',
    optional: true
  },
  position: {
    type: 'string',
    optional: true
  },
  vdi: {
    type: 'string'
  },
  vm: {
    type: 'string'
  }
};

attachDisk.resolve = {
  vm: ['vm', 'VM', 'administrate'],
  vdi: ['vdi', 'VDI', 'administrate']
};

exports.attachDisk = attachDisk;

createInterface = $coroutine(_regenerator2.default.mark(function _callee24(arg) {
  var allowedIpv4Addresses, allowedIpv6Addresses, ipAddresses, mac, network, position, push, vif, vm;
  return _regenerator2.default.wrap(function _callee24$(_context24) {
    while (1) {
      switch (_context24.prev = _context24.next) {
        case 0:
          vm = arg.vm, network = arg.network, position = arg.position, mac = arg.mac, allowedIpv4Addresses = arg.allowedIpv4Addresses, allowedIpv6Addresses = arg.allowedIpv6Addresses;
          _context24.next = 3;
          return this.getXapi(vm).createVif(vm._xapiId, network._xapiId, {
            mac: mac,
            position: position,
            ipv4_allowed: allowedIpv4Addresses,
            ipv6_allowed: allowedIpv6Addresses
          });

        case 3:
          vif = _context24.sent;

          push = (ipAddresses = []).push;
          if (allowedIpv4Addresses) {
            push.apply(ipAddresses, allowedIpv4Addresses);
          }
          if (allowedIpv6Addresses) {
            push.apply(ipAddresses, allowedIpv6Addresses);
          }
          if (ipAddresses.length) {
            pCatch.call(this.allocIpAddresses(vif.$id, allo), noop);
          }
          return _context24.abrupt('return', vif.$id);

        case 9:
        case 'end':
          return _context24.stop();
      }
    }
  }, _callee24, this);
}));

createInterface.params = {
  vm: {
    type: 'string'
  },
  network: {
    type: 'string'
  },
  position: {
    type: ['integer', 'string'],
    optional: true
  },
  mac: {
    type: 'string',
    optional: true
  },
  allowedIpv4Addresses: {
    type: 'array',
    items: {
      type: 'string'
    },
    optional: true
  },
  allowedIpv6Addresses: {
    type: 'array',
    items: {
      type: 'string'
    },
    optional: true
  }
};

createInterface.resolve = {
  vm: ['vm', 'VM', 'administrate'],
  network: ['network', 'network', 'view']
};

exports.createInterface = createInterface;

attachPci = $coroutine(_regenerator2.default.mark(function _callee25(arg) {
  var pciId, vm, xapi;
  return _regenerator2.default.wrap(function _callee25$(_context25) {
    while (1) {
      switch (_context25.prev = _context25.next) {
        case 0:
          vm = arg.vm, pciId = arg.pciId;
          xapi = this.getXapi(vm);
          _context25.next = 4;
          return xapi.call('VM.add_to_other_config', vm._xapiRef, 'pci', pciId);

        case 4:
          return _context25.abrupt('return', true);

        case 5:
        case 'end':
          return _context25.stop();
      }
    }
  }, _callee25, this);
}));

attachPci.params = {
  vm: {
    type: 'string'
  },
  pciId: {
    type: 'string'
  }
};

attachPci.resolve = {
  vm: ['vm', 'VM', 'administrate']
};

exports.attachPci = attachPci;

detachPci = $coroutine(_regenerator2.default.mark(function _callee26(arg) {
  var vm, xapi;
  return _regenerator2.default.wrap(function _callee26$(_context26) {
    while (1) {
      switch (_context26.prev = _context26.next) {
        case 0:
          vm = arg.vm;
          xapi = this.getXapi(vm);
          _context26.next = 4;
          return xapi.call('VM.remove_from_other_config', vm._xapiRef, 'pci');

        case 4:
          return _context26.abrupt('return', true);

        case 5:
        case 'end':
          return _context26.stop();
      }
    }
  }, _callee26, this);
}));

detachPci.params = {
  vm: {
    type: 'string'
  }
};

detachPci.resolve = {
  vm: ['vm', 'VM', 'administrate']
};

exports.detachPci = detachPci;

stats = $coroutine(_regenerator2.default.mark(function _callee27(arg) {
  var granularity, vm;
  return _regenerator2.default.wrap(function _callee27$(_context27) {
    while (1) {
      switch (_context27.prev = _context27.next) {
        case 0:
          vm = arg.vm, granularity = arg.granularity;
          _context27.next = 3;
          return this.getXapiVmStats(vm, granularity);

        case 3:
          stats = _context27.sent;
          return _context27.abrupt('return', stats);

        case 5:
        case 'end':
          return _context27.stop();
      }
    }
  }, _callee27, this);
}));

stats.description = 'returns statistics about the VM';

stats.params = {
  id: {
    type: 'string'
  },
  granularity: {
    type: 'string',
    optional: true
  }
};

stats.resolve = {
  vm: ['id', ['VM', 'VM-snapshot'], 'view']
};

exports.stats = stats;

setBootOrder = $coroutine(_regenerator2.default.mark(function _callee28(arg) {
  var order, vm, xapi;
  return _regenerator2.default.wrap(function _callee28$(_context28) {
    while (1) {
      switch (_context28.prev = _context28.next) {
        case 0:
          vm = arg.vm, order = arg.order;
          xapi = this.getXapi(vm);
          order = {
            order: order
          };

          if (!(vm.virtualizationMode === 'hvm')) {
            _context28.next = 7;
            break;
          }

          _context28.next = 6;
          return xapi.call('VM.set_HVM_boot_params', vm._xapiRef, order);

        case 6:
          return _context28.abrupt('return', true);

        case 7:
          throw invalidParameters('You can only set the boot order on a HVM guest');

        case 8:
        case 'end':
          return _context28.stop();
      }
    }
  }, _callee28, this);
}));

setBootOrder.params = {
  vm: {
    type: 'string'
  },
  order: {
    type: 'string'
  }
};

setBootOrder.resolve = {
  vm: ['vm', 'VM', 'operate']
};

exports.setBootOrder = setBootOrder;

recoveryStart = function recoveryStart(arg) {
  var vm;
  vm = arg.vm;
  return this.getXapi(vm).startVmOnCd(vm._xapiId);
};

recoveryStart.params = {
  id: {
    type: 'string'
  }
};

recoveryStart.resolve = {
  vm: ['id', 'VM', 'operate']
};

exports.recoveryStart = recoveryStart;

getCloudInitConfig = $coroutine(_regenerator2.default.mark(function _callee29(arg) {
  var template;
  return _regenerator2.default.wrap(function _callee29$(_context29) {
    while (1) {
      switch (_context29.prev = _context29.next) {
        case 0:
          template = arg.template;
          _context29.next = 3;
          return this.getXapi(template).getCloudInitConfig(template._xapiId);

        case 3:
          return _context29.abrupt('return', _context29.sent);

        case 4:
        case 'end':
          return _context29.stop();
      }
    }
  }, _callee29, this);
}));

getCloudInitConfig.params = {
  template: {
    type: 'string'
  }
};

getCloudInitConfig.resolve = {
  template: ['template', 'VM-template', 'administrate']
};

exports.getCloudInitConfig = getCloudInitConfig;

createCloudInitConfigDrive = $coroutine(_regenerator2.default.mark(function _callee30(arg) {
  var config, coreos, sr, vm, xapi;
  return _regenerator2.default.wrap(function _callee30$(_context30) {
    while (1) {
      switch (_context30.prev = _context30.next) {
        case 0:
          vm = arg.vm, sr = arg.sr, config = arg.config, coreos = arg.coreos;
          xapi = this.getXapi(vm);

          if (!coreos) {
            _context30.next = 7;
            break;
          }

          _context30.next = 5;
          return xapi.createCoreOsCloudInitConfigDrive(vm._xapiId, sr._xapiId, config);

        case 5:
          _context30.next = 9;
          break;

        case 7:
          _context30.next = 9;
          return xapi.createCloudInitConfigDrive(vm._xapiId, sr._xapiId, config);

        case 9:
          return _context30.abrupt('return', true);

        case 10:
        case 'end':
          return _context30.stop();
      }
    }
  }, _callee30, this);
}));

createCloudInitConfigDrive.params = {
  vm: {
    type: 'string'
  },
  sr: {
    type: 'string'
  },
  config: {
    type: 'string'
  }
};

createCloudInitConfigDrive.resolve = {
  vm: ['vm', 'VM', 'administrate'],
  sr: ['sr', 'SR', '']
};

exports.createCloudInitConfigDrive = createCloudInitConfigDrive;

Object.defineProperty(exports, '__esModule', {
  value: true
});
//# sourceMappingURL=vm.js.map
