'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.downloadAndInstallXosanPack = exports.computeXosanPossibleOptions = exports.createSR = exports.getVolumeInfo = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _includes2 = require('lodash/includes');

var _includes3 = _interopRequireDefault(_includes2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var getVolumeInfo = exports.getVolumeInfo = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(_ref2) {
    var sr = _ref2.sr;
    var xapi, giantIPtoVMDict, data, nodes, oneHostAndVm, resultCmd, result, info, getNumber, brickKeys, entry, brick;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            xapi = this.getXapi(sr);
            giantIPtoVMDict = {};
            data = xapi.xo.getData(sr, 'xosan_config');

            if (!(!data || !data.nodes)) {
              _context.next = 5;
              break;
            }

            return _context.abrupt('return', null);

          case 5:
            nodes = data.nodes;

            nodes.forEach(function (conf) {
              giantIPtoVMDict[conf.vm.ip] = xapi.getObject(conf.vm.id);
            });
            oneHostAndVm = nodes[0];
            _context.next = 10;
            return remoteSsh(xapi, {
              host: xapi.getObject(oneHostAndVm.host),
              address: oneHostAndVm.vm.ip
            }, 'gluster volume info xosan');

          case 10:
            resultCmd = _context.sent;
            result = resultCmd['stdout'];

            /*
             Volume Name: xosan
             Type: Disperse
             Volume ID: 1d4d0e57-8b6b-43f9-9d40-c48be1df7548
             Status: Started
             Snapshot Count: 0
             Number of Bricks: 1 x (2 + 1) = 3
             Transport-type: tcp
             Bricks:
             Brick1: 192.168.0.201:/bricks/brick1/xosan1
             Brick2: 192.168.0.202:/bricks/brick1/xosan1
             Brick3: 192.168.0.203:/bricks/brick1/xosan1
             Options Reconfigured:
             client.event-threads: 16
             server.event-threads: 16
             performance.client-io-threads: on
             nfs.disable: on
             performance.readdir-ahead: on
             transport.address-family: inet
             features.shard: on
             features.shard-block-size: 64MB
             network.remote-dio: enable
             cluster.eager-lock: enable
             performance.io-cache: off
             performance.read-ahead: off
             performance.quick-read: off
             performance.stat-prefetch: on
             performance.strict-write-ordering: off
             cluster.server-quorum-type: server
             cluster.quorum-type: auto
             */

            info = (0, _fromPairs2.default)((0, _splitLines2.default)(result.trim()).map(function (line) {
              return (0, _utils.splitFirst)(line, ':').map(function (val) {
                return val.trim();
              });
            }));

            getNumber = function getNumber(item) {
              return +item.substr(5);
            };

            brickKeys = (0, _filter3.default)((0, _keys2.default)(info), function (key) {
              return key.match(/^Brick[1-9]/);
            }).sort(function (i1, i2) {
              return getNumber(i1) - getNumber(i2);
            });

            // expected brickKeys : [ 'Brick1', 'Brick2', 'Brick3' ]

            info['Bricks'] = brickKeys.map(function (key) {
              var ip = info[key].split(':')[0];
              return { config: info[key], ip: ip, vm: giantIPtoVMDict[ip] };
            });
            _context.next = 18;
            return (0, _utils.pFromCallback)(function (cb) {
              return _arpA2.default.table(cb);
            });

          case 18:
            entry = _context.sent;

            if (entry) {
              brick = info['Bricks'].find(function (element) {
                return element.config.split(':')[0] === entry.ip;
              });

              if (brick) {
                brick.mac = entry.mac;
              }
            }

            return _context.abrupt('return', info);

          case 21:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getVolumeInfo(_x) {
    return _ref.apply(this, arguments);
  };
}();

var copyVm = function () {
  var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(xapi, originalVm, params) {
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return xapi.copyVm(originalVm, params.sr);

          case 2:
            _context2.t0 = _context2.sent;
            _context2.t1 = params;
            return _context2.abrupt('return', {
              vm: _context2.t0,
              params: _context2.t1
            });

          case 5:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function copyVm(_x2, _x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

var prepareGlusterVm = function () {
  var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(xapi, vmAndParam, xosanNetwork) {
    var increaseDataDisk = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;
    var vm, params, ip, sr, host, firstVif, dataDisk, srFreeSpace, newSize, vmIsUp;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            vm = vmAndParam.vm;
            // refresh the object so that sizes are correct

            params = vmAndParam.params;
            ip = params.xenstore_data['vm-data/ip'];
            sr = xapi.getObject(params.sr.$id);
            _context3.next = 6;
            return xapi._waitObjectState(sr.$id, function (sr) {
              return Boolean(sr.$PBDs);
            });

          case 6:
            host = sr.$PBDs[0].$host;
            firstVif = vm.$VIFs[0];

            if (!(xosanNetwork.$id !== firstVif.$network.$id)) {
              _context3.next = 11;
              break;
            }

            _context3.next = 11;
            return xapi.call('VIF.move', firstVif.$ref, xosanNetwork.$ref);

          case 11:
            _context3.next = 13;
            return xapi.editVm(vm, {
              name_label: params.name_label,
              name_description: params.name_description
            });

          case 13:
            _context3.next = 15;
            return xapi.call('VM.set_xenstore_data', vm.$ref, params.xenstore_data);

          case 15:
            if (!increaseDataDisk) {
              _context3.next = 21;
              break;
            }

            dataDisk = vm.$VBDs.map(function (vbd) {
              return vbd.$VDI;
            }).find(function (vdi) {
              return vdi && vdi.name_label === 'xosan_data';
            });
            srFreeSpace = sr.physical_size - sr.physical_utilisation;
            // we use a percentage because it looks like the VDI overhead is proportional

            newSize = floor2048((srFreeSpace + dataDisk.virtual_size) * XOSAN_DATA_DISK_USEAGE_RATIO);
            _context3.next = 21;
            return xapi._resizeVdi(dataDisk, Math.min(newSize, XOSAN_MAX_DISK_SIZE));

          case 21:
            _context3.next = 23;
            return xapi.startVm(vm);

          case 23:
            debug('waiting for boot of ', ip);
            // wait until we find the assigned IP in the networks, we are just checking the boot is complete

            vmIsUp = function vmIsUp(vm) {
              return Boolean(vm.$guest_metrics && (0, _includes3.default)(vm.$guest_metrics.networks, ip));
            };

            _context3.next = 27;
            return xapi._waitObjectState(vm.$id, vmIsUp);

          case 27:
            vm = _context3.sent;

            debug('booted ', ip);
            return _context3.abrupt('return', { address: ip, host, vm });

          case 30:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function prepareGlusterVm(_x5, _x6, _x7) {
    return _ref4.apply(this, arguments);
  };
}();

var callPlugin = function () {
  var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(xapi, host, command, params) {
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            debug('calling plugin', host.address, command);
            _context4.t0 = JSON;
            _context4.next = 4;
            return xapi.call('host.call_plugin', host.$ref, 'xosan.py', command, params);

          case 4:
            _context4.t1 = _context4.sent;
            return _context4.abrupt('return', _context4.t0.parse.call(_context4.t0, _context4.t1));

          case 6:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function callPlugin(_x9, _x10, _x11, _x12) {
    return _ref5.apply(this, arguments);
  };
}();

var remoteSsh = function () {
  var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5(xapi, hostAndAddress, cmd) {
    var result;
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return callPlugin(xapi, hostAndAddress.host, 'run_ssh', {
              destination: 'root@' + hostAndAddress.address,
              cmd: cmd
            });

          case 2:
            result = _context5.sent;

            if (!(result.exit !== 0)) {
              _context5.next = 5;
              break;
            }

            throw new Error('ssh error: ' + (0, _stringify2.default)(result));

          case 5:
            debug(result);
            return _context5.abrupt('return', result);

          case 7:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));

  return function remoteSsh(_x13, _x14, _x15) {
    return _ref6.apply(this, arguments);
  };
}();

var setPifIp = function () {
  var _ref7 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6(xapi, pif, address) {
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return xapi.call('PIF.reconfigure_ip', pif.$ref, 'Static', address, '255.255.255.0', NETWORK_PREFIX + '1', '');

          case 2:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));

  return function setPifIp(_x16, _x17, _x18) {
    return _ref7.apply(this, arguments);
  };
}();

var getOrCreateSshKey = function () {
  var _ref9 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee9(xapi) {
    var _this = this;

    var sshKey, readKeys;
    return _regenerator2.default.wrap(function _callee9$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            sshKey = xapi.xo.getData(xapi.pool, 'xosan_ssh_key');

            if (sshKey) {
              _context10.next = 14;
              break;
            }

            readKeys = function () {
              var _ref10 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8() {
                return _regenerator2.default.wrap(function _callee8$(_context9) {
                  while (1) {
                    switch (_context9.prev = _context9.next) {
                      case 0:
                        _context9.next = 2;
                        return _fsPromise2.default.readFile(SSH_KEY_FILE, 'ascii');

                      case 2:
                        _context9.t0 = _context9.sent;
                        _context9.next = 5;
                        return _fsPromise2.default.readFile(SSH_KEY_FILE + '.pub', 'ascii');

                      case 5:
                        _context9.t1 = _context9.sent;
                        sshKey = {
                          private: _context9.t0,
                          public: _context9.t1
                        };

                        xapi.xo.setData(xapi.pool, 'xosan_ssh_key', sshKey);

                      case 8:
                      case 'end':
                        return _context9.stop();
                    }
                  }
                }, _callee8, _this);
              }));

              return function readKeys() {
                return _ref10.apply(this, arguments);
              };
            }();

            _context10.prev = 3;
            _context10.next = 6;
            return readKeys();

          case 6:
            _context10.next = 14;
            break;

          case 8:
            _context10.prev = 8;
            _context10.t0 = _context10['catch'](3);
            _context10.next = 12;
            return (0, _execa2.default)('ssh-keygen', ['-q', '-f', SSH_KEY_FILE, '-t', 'rsa', '-b', '4096', '-N', '']);

          case 12:
            _context10.next = 14;
            return readKeys();

          case 14:
            return _context10.abrupt('return', sshKey);

          case 15:
          case 'end':
            return _context10.stop();
        }
      }
    }, _callee9, this, [[3, 8]]);
  }));

  return function getOrCreateSshKey(_x23) {
    return _ref9.apply(this, arguments);
  };
}();

var configureGluster = function () {
  var _ref11 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee10(redundancy, ipAndHosts, xapi, firstIpAndHost, glusterType) {
    var arbiter = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : null;

    var configByType, brickVms, i, creation, volumeCreation, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, confChunk;

    return _regenerator2.default.wrap(function _callee10$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            configByType = {
              replica_arbiter: {
                creation: 'replica 3 arbiter 1',
                extra: []
              },
              replica: {
                creation: 'replica ' + redundancy + ' ',
                extra: ['gluster volume set xosan cluster.data-self-heal on']
              },
              disperse: {
                creation: 'disperse ' + ipAndHosts.length + ' redundancy ' + redundancy + ' ',
                extra: []
              }
            };
            brickVms = arbiter ? ipAndHosts.concat(arbiter) : ipAndHosts;
            i = 1;

          case 3:
            if (!(i < brickVms.length)) {
              _context11.next = 9;
              break;
            }

            _context11.next = 6;
            return remoteSsh(xapi, firstIpAndHost, 'gluster peer probe ' + brickVms[i].address);

          case 6:
            i++;
            _context11.next = 3;
            break;

          case 9:
            creation = configByType[glusterType].creation;
            volumeCreation = 'gluster volume create xosan ' + creation + ' ' + brickVms.map(function (ipAndHost) {
              return ipAndHost.address + ':/bricks/xosan/xosandir';
            }).join(' ');

            debug('creating volume: ', volumeCreation);
            _context11.next = 14;
            return remoteSsh(xapi, firstIpAndHost, volumeCreation);

          case 14:
            _context11.next = 16;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan network.remote-dio enable');

          case 16:
            _context11.next = 18;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan cluster.eager-lock enable');

          case 18:
            _context11.next = 20;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan performance.io-cache off');

          case 20:
            _context11.next = 22;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan performance.read-ahead off');

          case 22:
            _context11.next = 24;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan performance.quick-read off');

          case 24:
            _context11.next = 26;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan performance.strict-write-ordering off');

          case 26:
            _context11.next = 28;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan client.event-threads 8');

          case 28:
            _context11.next = 30;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan server.event-threads 8');

          case 30:
            _context11.next = 32;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan performance.io-thread-count 64');

          case 32:
            _context11.next = 34;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan performance.stat-prefetch on');

          case 34:
            _context11.next = 36;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan features.shard on');

          case 36:
            _context11.next = 38;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume set xosan features.shard-block-size 512MB');

          case 38:
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context11.prev = 41;
            _iterator = (0, _getIterator3.default)(configByType[glusterType].extra);

          case 43:
            if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
              _context11.next = 50;
              break;
            }

            confChunk = _step.value;
            _context11.next = 47;
            return remoteSsh(xapi, firstIpAndHost, confChunk);

          case 47:
            _iteratorNormalCompletion = true;
            _context11.next = 43;
            break;

          case 50:
            _context11.next = 56;
            break;

          case 52:
            _context11.prev = 52;
            _context11.t0 = _context11['catch'](41);
            _didIteratorError = true;
            _iteratorError = _context11.t0;

          case 56:
            _context11.prev = 56;
            _context11.prev = 57;

            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }

          case 59:
            _context11.prev = 59;

            if (!_didIteratorError) {
              _context11.next = 62;
              break;
            }

            throw _iteratorError;

          case 62:
            return _context11.finish(59);

          case 63:
            return _context11.finish(56);

          case 64:
            _context11.next = 66;
            return remoteSsh(xapi, firstIpAndHost, 'gluster volume start xosan');

          case 66:
          case 'end':
            return _context11.stop();
        }
      }
    }, _callee10, this, [[41, 52, 56, 64], [57,, 59, 63]]);
  }));

  return function configureGluster(_x24, _x25, _x26, _x27, _x28) {
    return _ref11.apply(this, arguments);
  };
}();

var computeXosanPossibleOptions = exports.computeXosanPossibleOptions = function () {
  var _ref15 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee12(_ref16) {
    var lvmSrs = _ref16.lvmSrs;
    var count, configurations, xapi, srs, srSizes, minSize, brickSize;
    return _regenerator2.default.wrap(function _callee12$(_context18) {
      while (1) {
        switch (_context18.prev = _context18.next) {
          case 0:
            count = lvmSrs.length;
            configurations = POSSIBLE_CONFIGURATIONS[count];

            if (configurations) {
              _context18.next = 4;
              break;
            }

            return _context18.abrupt('return', null);

          case 4:
            if (!(count > 0)) {
              _context18.next = 11;
              break;
            }

            xapi = this.getXapi(lvmSrs[0]);
            srs = (0, _map2.default)(lvmSrs, function (srId) {
              return xapi.getObject(srId);
            });
            srSizes = (0, _map2.default)(srs, function (sr) {
              return sr.physical_size - sr.physical_utilisation;
            });
            minSize = Math.min.apply(null, srSizes);
            brickSize = (minSize - XOSAN_VM_SYSTEM_DISK_SIZE) * XOSAN_DATA_DISK_USEAGE_RATIO;
            return _context18.abrupt('return', configurations.map(function (conf) {
              return (0, _extends3.default)({}, conf, { availableSpace: brickSize * conf.capacity });
            }));

          case 11:
          case 'end':
            return _context18.stop();
        }
      }
    }, _callee12, this);
  }));

  return function computeXosanPossibleOptions(_x32) {
    return _ref15.apply(this, arguments);
  };
}();

// ---------------------------------------------------------------------

var downloadAndInstallXosanPack = exports.downloadAndInstallXosanPack = function () {
  var _ref17 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee13(_ref18) {
    var id = _ref18.id,
        version = _ref18.version,
        pool = _ref18.pool;
    var xapi, res;
    return _regenerator2.default.wrap(function _callee13$(_context19) {
      while (1) {
        switch (_context19.prev = _context19.next) {
          case 0:
            if (this.requestResource) {
              _context19.next = 2;
              break;
            }

            throw new Error('requestResource is not a function');

          case 2:
            xapi = this.getXapi(pool.id);
            _context19.next = 5;
            return this.requestResource('xosan', id, version);

          case 5:
            res = _context19.sent;
            return _context19.abrupt('return', xapi.installSupplementalPackOnAllHosts(res));

          case 7:
          case 'end':
            return _context19.stop();
        }
      }
    }, _callee13, this);
  }));

  return function downloadAndInstallXosanPack(_x33) {
    return _ref17.apply(this, arguments);
  };
}();

exports.checkSrIsBusy = checkSrIsBusy;

var _arpA = require('arp-a');

var _arpA2 = _interopRequireDefault(_arpA);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _golikeDefer = require('golike-defer');

var _golikeDefer2 = _interopRequireDefault(_golikeDefer);

var _execa = require('execa');

var _execa2 = _interopRequireDefault(_execa);

var _fromPairs = require('lodash/fromPairs');

var _fromPairs2 = _interopRequireDefault(_fromPairs);

var _fsPromise = require('fs-promise');

var _fsPromise2 = _interopRequireDefault(_fsPromise);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _splitLines = require('split-lines');

var _splitLines2 = _interopRequireDefault(_splitLines);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('xo:xosan');

var SSH_KEY_FILE = 'id_rsa_xosan';
var NETWORK_PREFIX = '172.31.100.';

var XOSAN_VM_SYSTEM_DISK_SIZE = 10 * 1024 * 1024 * 1024;
var XOSAN_DATA_DISK_USEAGE_RATIO = 0.99;
var XOSAN_MAX_DISK_SIZE = 2093050 * 1024 * 1024; // a bit under 2To

var CURRENTLY_CREATING_SRS = {};

getVolumeInfo.description = 'info on gluster volume';
getVolumeInfo.permission = 'admin';

getVolumeInfo.params = {
  sr: {
    type: 'string'
  }
};
getVolumeInfo.resolve = {
  sr: ['sr', 'SR', 'administrate']
};
function floor2048(value) {
  return 2048 * Math.floor(value / 2048);
}

var createNetworkAndInsertHosts = _golikeDefer2.default.onFailure(function () {
  var _ref8 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7($onFailure, xapi, pif, vlan) {
    var hostIpLastNumber, xosanNetwork;
    return _regenerator2.default.wrap(function _callee7$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            hostIpLastNumber = 1;
            _context8.next = 3;
            return xapi.createNetwork({
              name: 'XOSAN network',
              description: 'XOSAN network',
              pifId: pif._xapiId,
              mtu: 9000,
              vlan: +vlan
            });

          case 3:
            xosanNetwork = _context8.sent;

            $onFailure(function () {
              var _context7;

              return (_context7 = xapi.deleteNetwork(xosanNetwork), _utils.pCatch).call(_context7, _utils.noop);
            });
            _context8.next = 7;
            return _promise2.default.all(xosanNetwork.$PIFs.map(function (pif) {
              return setPifIp(xapi, pif, NETWORK_PREFIX + hostIpLastNumber++);
            }));

          case 7:
            return _context8.abrupt('return', xosanNetwork);

          case 8:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee7, this);
  }));

  return function (_x19, _x20, _x21, _x22) {
    return _ref8.apply(this, arguments);
  };
}());
var createSR = exports.createSR = _golikeDefer2.default.onFailure(function () {
  var _ref12 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee11($onFailure, _ref13) {
    var template = _ref13.template,
        pif = _ref13.pif,
        vlan = _ref13.vlan,
        srs = _ref13.srs,
        glusterType = _ref13.glusterType,
        redundancy = _ref13.redundancy;
    var vmIpLastNumber, xapi, xosanNetwork, sshKey, srsObjects, vmParameters, firstVM, copiedVms, vmsAndParams, arbiter, sr, arbiterConfig, arbiterVm, ipAndHosts, firstIpAndHost, config, xosanSr;
    return _regenerator2.default.wrap(function _callee11$(_context17) {
      while (1) {
        switch (_context17.prev = _context17.next) {
          case 0:
            if (this.requestResource) {
              _context17.next = 2;
              break;
            }

            throw new Error('requestResource is not a function');

          case 2:
            if (!(srs.length < 1)) {
              _context17.next = 4;
              break;
            }

            return _context17.abrupt('return');

          case 4:
            vmIpLastNumber = 101;
            xapi = this.getXapi(srs[0]);

            if (!CURRENTLY_CREATING_SRS[xapi.pool.$id]) {
              _context17.next = 8;
              break;
            }

            throw new Error('createSR is already running for this pool');

          case 8:

            CURRENTLY_CREATING_SRS[xapi.pool.$id] = true;
            _context17.prev = 9;
            _context17.next = 12;
            return createNetworkAndInsertHosts(xapi, pif, vlan);

          case 12:
            xosanNetwork = _context17.sent;

            $onFailure(function () {
              var _context12;

              return (_context12 = xapi.deleteNetwork(xosanNetwork), _utils.pCatch).call(_context12, _utils.noop);
            });
            _context17.next = 16;
            return getOrCreateSshKey(xapi);

          case 16:
            sshKey = _context17.sent;
            srsObjects = (0, _map2.default)(srs, function (srId) {
              return xapi.getObject(srId);
            });
            vmParameters = (0, _map2.default)(srs, function (srId) {
              var sr = xapi.getObject(srId);
              var host = sr.$PBDs[0].$host;
              return {
                sr,
                host,
                name_label: `XOSAN - ${sr.name_label} - ${host.name_label}`,
                name_description: 'Xosan VM storing data on volume ' + sr.name_label,
                // the values of the xenstore_data object *have* to be string, don't forget.
                xenstore_data: {
                  'vm-data/hostname': 'XOSAN' + sr.name_label,
                  'vm-data/sshkey': sshKey.public,
                  'vm-data/ip': NETWORK_PREFIX + vmIpLastNumber++,
                  'vm-data/mtu': String(xosanNetwork.MTU),
                  'vm-data/vlan': String(vlan)
                }
              };
            });
            _context17.next = 21;
            return _promise2.default.all(vmParameters.map(function (vmParam) {
              return callPlugin(xapi, vmParam.host, 'receive_ssh_keys', {
                private_key: sshKey.private,
                public_key: sshKey.public,
                force: 'true'
              });
            }));

          case 21:
            _context17.t0 = xapi;
            _context17.next = 24;
            return this.requestResource('xosan', template.id, template.version);

          case 24:
            _context17.t1 = _context17.sent;
            _context17.t2 = { srId: vmParameters[0].sr.$ref, type: 'xva' };
            _context17.next = 28;
            return _context17.t0.importVm.call(_context17.t0, _context17.t1, _context17.t2);

          case 28:
            firstVM = _context17.sent;

            $onFailure(function () {
              var _context13;

              return (_context13 = xapi.deleteVm(firstVM, true), _utils.pCatch).call(_context13, _utils.noop);
            });
            _context17.next = 32;
            return xapi.editVm(firstVM, {
              autoPoweron: true
            });

          case 32:
            _context17.next = 34;
            return _promise2.default.all(vmParameters.slice(1).map(function (param) {
              return copyVm(xapi, firstVM, param);
            }));

          case 34:
            copiedVms = _context17.sent;

            // TODO: Promise.all() is certainly not the right operation to execute all the given promises whether they fulfill or reject.
            $onFailure(function () {
              var _context14;

              return (_context14 = _promise2.default.all(copiedVms.map(function (vm) {
                return xapi.deleteVm(vm.vm, true);
              })), _utils.pCatch).call(_context14, _utils.noop);
            });
            vmsAndParams = [{
              vm: firstVM,
              params: vmParameters[0]
            }].concat(copiedVms);
            arbiter = null;

            if (!(srs.length === 2)) {
              _context17.next = 48;
              break;
            }

            sr = vmParameters[0].sr;
            arbiterConfig = {
              sr: sr,
              host: vmParameters[0].host,
              name_label: vmParameters[0].name_label + ' arbiter',
              name_description: 'Xosan VM storing data on volume ' + sr.name_label,
              xenstore_data: {
                'vm-data/hostname': 'XOSAN' + sr.name_label + '_arb',
                'vm-data/sshkey': sshKey.public,
                'vm-data/ip': NETWORK_PREFIX + vmIpLastNumber++,
                'vm-data/mtu': String(xosanNetwork.MTU),
                'vm-data/vlan': String(vlan)
              }
            };
            _context17.next = 43;
            return copyVm(xapi, firstVM, arbiterConfig);

          case 43:
            arbiterVm = _context17.sent;

            $onFailure(function () {
              var _context15;

              return (_context15 = xapi.deleteVm(arbiterVm.vm, true), _utils.pCatch).call(_context15, _utils.noop);
            });
            _context17.next = 47;
            return prepareGlusterVm(xapi, arbiterVm, xosanNetwork, false);

          case 47:
            arbiter = _context17.sent;

          case 48:
            _context17.next = 50;
            return _promise2.default.all((0, _map2.default)(vmsAndParams, function (vmAndParam) {
              return prepareGlusterVm(xapi, vmAndParam, xosanNetwork);
            }));

          case 50:
            ipAndHosts = _context17.sent;
            firstIpAndHost = ipAndHosts[0];
            _context17.next = 54;
            return configureGluster(redundancy, ipAndHosts, xapi, firstIpAndHost, glusterType, arbiter);

          case 54:
            debug('xosan gluster volume started');
            config = { server: firstIpAndHost.address + ':/xosan', backupserver: ipAndHosts[1].address };
            _context17.next = 58;
            return xapi.call('SR.create', srsObjects[0].$PBDs[0].$host.$ref, config, 0, 'XOSAN', 'XOSAN', 'xosan', '', true, {});

          case 58:
            xosanSr = _context17.sent;

            if (arbiter) {
              ipAndHosts.push(arbiter);
            }
            // we just forget because the cleanup actions will be executed before.
            $onFailure(function () {
              var _context16;

              return (_context16 = xapi.forgetSr(xosanSr), _utils.pCatch).call(_context16, _utils.noop);
            });
            _context17.next = 63;
            return xapi.xo.setData(xosanSr, 'xosan_config', {
              nodes: ipAndHosts.map(function (param) {
                return {
                  host: param.host.$id,
                  vm: { id: param.vm.$id, ip: param.address }
                };
              }),
              network: xosanNetwork.$id
            });

          case 63:
            _context17.prev = 63;

            delete CURRENTLY_CREATING_SRS[xapi.pool.$id];
            return _context17.finish(63);

          case 66:
          case 'end':
            return _context17.stop();
        }
      }
    }, _callee11, this, [[9,, 63, 66]]);
  }));

  return function (_x30, _x31) {
    return _ref12.apply(this, arguments);
  };
}());

createSR.description = 'create gluster VM';
createSR.permission = 'admin';
createSR.params = {
  srs: {
    type: 'array',
    items: {
      type: 'string'
    }
  },
  pif: {
    type: 'string'
  },
  vlan: {
    type: 'string'
  },
  glusterType: {
    type: 'string'
  },
  redundancy: {
    type: 'number'
  }
};

createSR.resolve = {
  srs: ['sr', 'SR', 'administrate'],
  pif: ['pif', 'PIF', 'administrate']
};

function checkSrIsBusy(_ref14) {
  var poolId = _ref14.poolId;

  return !!CURRENTLY_CREATING_SRS[poolId];
}
checkSrIsBusy.description = 'checks if there is a xosan SR curently being created on the given pool id';
checkSrIsBusy.permission = 'admin';
checkSrIsBusy.params = { poolId: { type: 'string' } };

var POSSIBLE_CONFIGURATIONS = {};
POSSIBLE_CONFIGURATIONS[2] = [{ layout: 'replica_arbiter', redundancy: 3, capacity: 1 }];
POSSIBLE_CONFIGURATIONS[3] = [{ layout: 'disperse', redundancy: 1, capacity: 2 }, { layout: 'replica', redundancy: 3, capacity: 1 }];
POSSIBLE_CONFIGURATIONS[4] = [{ layout: 'replica', redundancy: 2, capacity: 1 }];
POSSIBLE_CONFIGURATIONS[5] = [{ layout: 'disperse', redundancy: 1, capacity: 4 }];
POSSIBLE_CONFIGURATIONS[6] = [{ layout: 'disperse', redundancy: 2, capacity: 4 }, { layout: 'replica', redundancy: 2, capacity: 3 }, { layout: 'replica', redundancy: 3, capacity: 2 }];
POSSIBLE_CONFIGURATIONS[7] = [{ layout: 'disperse', redundancy: 3, capacity: 4 }];
POSSIBLE_CONFIGURATIONS[8] = [{ layout: 'replica', redundancy: 2, capacity: 4 }];
POSSIBLE_CONFIGURATIONS[9] = [{ layout: 'disperse', redundancy: 1, capacity: 8 }, { layout: 'replica', redundancy: 3, capacity: 3 }];
POSSIBLE_CONFIGURATIONS[10] = [{ layout: 'disperse', redundancy: 2, capacity: 8 }, { layout: 'replica', redundancy: 2, capacity: 5 }];
POSSIBLE_CONFIGURATIONS[11] = [{ layout: 'disperse', redundancy: 3, capacity: 8 }];
POSSIBLE_CONFIGURATIONS[12] = [{ layout: 'disperse', redundancy: 4, capacity: 8 }, { layout: 'replica', redundancy: 2, capacity: 6 }];
POSSIBLE_CONFIGURATIONS[13] = [{ layout: 'disperse', redundancy: 5, capacity: 8 }];
POSSIBLE_CONFIGURATIONS[14] = [{ layout: 'disperse', redundancy: 6, capacity: 8 }, { layout: 'replica', redundancy: 2, capacity: 7 }];
POSSIBLE_CONFIGURATIONS[15] = [{ layout: 'disperse', redundancy: 7, capacity: 8 }, { layout: 'replica', redundancy: 3, capacity: 5 }];
POSSIBLE_CONFIGURATIONS[16] = [{ layout: 'replica', redundancy: 2, capacity: 8 }];

computeXosanPossibleOptions.params = {
  lvmSrs: {
    type: 'array',
    items: {
      type: 'string'
    }
  }
};

downloadAndInstallXosanPack.description = 'Register a resource via cloud plugin';

downloadAndInstallXosanPack.params = {
  id: { type: 'string' },
  version: { type: 'string' },
  pool: { type: 'string' }
};

downloadAndInstallXosanPack.resolve = {
  pool: ['pool', 'pool', 'administrate']
};

downloadAndInstallXosanPack.permission = 'admin';
//# sourceMappingURL=xosan.js.map
