'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _collection = require('../collection');

var _collection2 = _interopRequireDefault(_collection);

var _difference = require('lodash/difference');

var _difference2 = _interopRequireDefault(_difference);

var _filter = require('lodash/filter');

var _filter2 = _interopRequireDefault(_filter);

var _keys = require('lodash/keys');

var _keys2 = _interopRequireDefault(_keys);

var _redis = require('redis');

var _uuid = require('uuid');

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ===================================================================

// ///////////////////////////////////////////////////////////////////
// Data model:
// - prefix +'_id': value of the last generated identifier;
// - prefix +'_ids': set containing identifier of all models;
// - prefix +'_'+ index +':' + value: set of identifiers which have
//   value for the given index.
// - prefix +':'+ id: hash containing the properties of a model;
// ///////////////////////////////////////////////////////////////////

// TODO: then-redis sends commands in order, we should use this
// semantic to simplify the code.

// TODO: Merge the options in the object to obtain extend-time
// configuration like Backbone.

// TODO: Remote events.

var Redis = function (_Collection) {
  (0, _inherits3.default)(Redis, _Collection);

  function Redis(_ref) {
    var connection = _ref.connection,
        _ref$indexes = _ref.indexes,
        indexes = _ref$indexes === undefined ? [] : _ref$indexes,
        prefix = _ref.prefix,
        uri = _ref.uri;
    (0, _classCallCheck3.default)(this, Redis);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Redis.__proto__ || (0, _getPrototypeOf2.default)(Redis)).call(this));

    _this.indexes = indexes;
    _this.prefix = prefix;
    _this.redis = (0, _utils.promisifyAll)(connection || (0, _redis.createClient)(uri));
    return _this;
  }

  (0, _createClass3.default)(Redis, [{
    key: '_extract',
    value: function _extract(ids) {
      var prefix = this.prefix + ':';
      var redis = this.redis;


      var models = [];
      return _promise2.default.all((0, _utils.mapToArray)(ids, function (id) {
        return redis.hgetall(prefix + id).then(function (model) {
          // If empty, consider it a no match.
          if ((0, _utils.isEmpty)(model)) {
            return;
          }

          // Mix the identifier in.
          model.id = id;

          models.push(model);
        });
      })).then(function () {
        return models;
      });
    }
  }, {
    key: '_add',
    value: function _add(models) {
      var _this2 = this;

      var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref2$replace = _ref2.replace,
          replace = _ref2$replace === undefined ? false : _ref2$replace;

      // TODO: remove “replace” which is a temporary measure, implement
      // “set()” instead.

      var indexes = this.indexes,
          prefix = this.prefix,
          redis = this.redis;


      return _promise2.default.all((0, _utils.mapToArray)(models, function () {
        var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(model) {
          var success, params, key, promises;
          return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  // Generate a new identifier if necessary.
                  if (model.id === undefined) {
                    model.id = (0, _uuid.v4)();
                  }

                  _context.next = 3;
                  return redis.sadd(prefix + '_ids', model.id);

                case 3:
                  success = _context.sent;

                  if (!(!success && !replace)) {
                    _context.next = 6;
                    break;
                  }

                  throw new _collection.ModelAlreadyExists(model.id);

                case 6:

                  // TODO: Remove existing fields.

                  params = [];

                  (0, _utils.forEach)(model, function (value, name) {
                    // No need to store the identifier (already in the key).
                    if (name === 'id') {
                      return;
                    }

                    params.push(name, value);
                  });

                  key = `${prefix}:${model.id}`;
                  promises = [redis.del(key), redis.hmset.apply(redis, [key].concat(params))];

                  // Update indexes.

                  (0, _utils.forEach)(indexes, function (index) {
                    var value = model[index];
                    if (value === undefined) {
                      return;
                    }

                    var key = prefix + '_' + index + ':' + value;
                    promises.push(redis.sadd(key, model.id));
                  });

                  _context.next = 13;
                  return _promise2.default.all(promises);

                case 13:
                  return _context.abrupt('return', model);

                case 14:
                case 'end':
                  return _context.stop();
              }
            }
          }, _callee, _this2);
        }));

        return function (_x2) {
          return _ref3.apply(this, arguments);
        };
      }()));
    }
  }, {
    key: '_get',
    value: function _get(properties) {
      var _this3 = this;

      var prefix = this.prefix,
          redis = this.redis;


      if ((0, _utils.isEmpty)(properties)) {
        return redis.smembers(prefix + '_ids').then(function (ids) {
          return _this3._extract(ids);
        });
      }

      // Special treatment for the identifier.
      var id = properties.id;
      if (id !== undefined) {
        delete properties.id;
        return this._extract([id]).then(function (models) {
          return models.length && !(0, _utils.isEmpty)(properties) ? (0, _filter2.default)(models) : models;
        });
      }

      var indexes = this.indexes;

      // Check for non indexed fields.

      var unfit = (0, _difference2.default)((0, _keys2.default)(properties), indexes);
      if (unfit.length) {
        throw new Error('fields not indexed: ' + unfit.join());
      }

      var keys = (0, _utils.mapToArray)(properties, function (value, index) {
        return `${prefix}_${index}:${value}`;
      });
      return redis.sinter.apply(redis, (0, _toConsumableArray3.default)(keys)).then(function (ids) {
        return _this3._extract(ids);
      });
    }
  }, {
    key: '_remove',
    value: function _remove(ids) {
      if ((0, _utils.isEmpty)(ids)) {
        return;
      }

      var prefix = this.prefix,
          redis = this.redis;

      // TODO: handle indexes.

      return _promise2.default.all([
      // Remove the identifiers from the main index.
      redis.srem.apply(redis, [prefix + '_ids'].concat((0, _toConsumableArray3.default)(ids))),

      // Remove the models.
      redis.del((0, _utils.mapToArray)(ids, function (id) {
        return `${prefix}:${id}`;
      }))]);
    }
  }, {
    key: '_update',
    value: function _update(models) {
      return this._add(models, { replace: true });
    }
  }]);
  return Redis;
}(_collection2.default);

exports.default = Redis;
//# sourceMappingURL=redis.js.map
