'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mixin = exports.debounce = undefined;

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _ownKeys2 = require('babel-runtime/core-js/reflect/own-keys');

var _ownKeys3 = _interopRequireDefault(_ownKeys2);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

var _getOwnPropertyDescriptor = require('babel-runtime/core-js/object/get-own-property-descriptor');

var _getOwnPropertyDescriptor2 = _interopRequireDefault(_getOwnPropertyDescriptor);

var _defineProperties = require('babel-runtime/core-js/object/define-properties');

var _defineProperties2 = _interopRequireDefault(_defineProperties);

var _bindPropertyDescriptor = require('bind-property-descriptor');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ===================================================================

var defineProperties = _defineProperties2.default,
    getOwnPropertyDescriptor = _getOwnPropertyDescriptor2.default;

// ===================================================================

// Debounce decorator for methods.
//
// See: https://github.com/wycats/javascript-decorators
//
// TODO: make it work for single functions.

var debounce = exports.debounce = function debounce(duration) {
  return function (target, name, descriptor) {
    var fn = descriptor.value;

    // This symbol is used to store the related data directly on the
    // current object.
    var s = (0, _symbol2.default)(`debounced ${name} data`);

    function debounced() {
      var data = this[s] || (this[s] = {
        lastCall: 0,
        wrapper: null
      });

      var now = Date.now();
      if (now > data.lastCall + duration) {
        data.lastCall = now;
        try {
          var result = fn.apply(this, arguments);
          data.wrapper = function () {
            return result;
          };
        } catch (error) {
          data.wrapper = function () {
            throw error;
          };
        }
      }
      return data.wrapper();
    }
    debounced.reset = function (obj) {
      delete obj[s];
    };

    descriptor.value = debounced;
    return descriptor;
  };
};

// -------------------------------------------------------------------

var _ownKeys = typeof Reflect !== 'undefined' && _ownKeys3.default || function (_ref) {
  var names = _ref.getOwnPropertyNames,
      symbols = _ref.getOwnPropertySymbols;
  return symbols ? function (obj) {
    return names(obj).concat(symbols(obj));
  } : names;
}(Object);

var _isIgnoredProperty = function _isIgnoredProperty(name) {
  return name[0] === '_' || name === 'constructor';
};

var _IGNORED_STATIC_PROPERTIES = {
  __proto__: null,

  arguments: true,
  caller: true,
  length: true,
  name: true,
  prototype: true
};
var _isIgnoredStaticProperty = function _isIgnoredStaticProperty(name) {
  return _IGNORED_STATIC_PROPERTIES[name];
};

var mixin = exports.mixin = function mixin(MixIns) {
  return function (Class) {
    if (!(0, _utils.isArray)(MixIns)) {
      MixIns = [MixIns];
    }

    var name = Class.name;

    // Copy properties of plain object mix-ins to the prototype.

    {
      var allMixIns = MixIns;
      MixIns = [];
      var prototype = Class.prototype;

      var _descriptors = { __proto__: null };
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(allMixIns), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var MixIn = _step.value;

          if ((0, _utils.isFunction)(MixIn)) {
            MixIns.push(MixIn);
            continue;
          }

          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = (0, _getIterator3.default)(_ownKeys(MixIn)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var prop = _step2.value;

              if (prop in prototype) {
                throw new Error(`${name}#${prop} is already defined`);
              }

              (_descriptors[prop] = getOwnPropertyDescriptor(MixIn, prop)).enumerable = false; // Object methods are enumerable but class methods are not.
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      defineProperties(prototype, _descriptors);
    }

    function Decorator() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var instance = new (Function.prototype.bind.apply(Class, [null].concat(args)))();

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = (0, _getIterator3.default)(MixIns), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var _MixIn = _step3.value;
          var _prototype = _MixIn.prototype;

          var mixinInstance = new _MixIn(instance);
          var _descriptors2 = { __proto__: null };
          var _iteratorNormalCompletion4 = true;
          var _didIteratorError4 = false;
          var _iteratorError4 = undefined;

          try {
            for (var _iterator4 = (0, _getIterator3.default)(_ownKeys(_prototype)), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
              var _prop = _step4.value;

              if (_isIgnoredProperty(_prop)) {
                continue;
              }

              if (_prop in instance) {
                throw new Error(`${name}#${_prop} is already defined`);
              }

              _descriptors2[_prop] = (0, _bindPropertyDescriptor.getBoundPropertyDescriptor)(_prototype, _prop, mixinInstance);
            }
          } catch (err) {
            _didIteratorError4 = true;
            _iteratorError4 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
              }
            } finally {
              if (_didIteratorError4) {
                throw _iteratorError4;
              }
            }
          }

          defineProperties(instance, _descriptors2);
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      return instance;
    }

    // Copy original and mixed-in static properties on Decorator class.
    var descriptors = { __proto__: null };
    var _iteratorNormalCompletion5 = true;
    var _didIteratorError5 = false;
    var _iteratorError5 = undefined;

    try {
      for (var _iterator5 = (0, _getIterator3.default)(_ownKeys(Class)), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
        var _prop2 = _step5.value;

        var descriptor = void 0;
        if (!(
        // Special properties are not defined...
        _isIgnoredStaticProperty(_prop2) && (

        // if they already exist...
        descriptor = getOwnPropertyDescriptor(Decorator, _prop2)) &&

        // and are not configurable.
        !descriptor.configurable)) {
          descriptors[_prop2] = getOwnPropertyDescriptor(Class, _prop2);
        }
      }
    } catch (err) {
      _didIteratorError5 = true;
      _iteratorError5 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion5 && _iterator5.return) {
          _iterator5.return();
        }
      } finally {
        if (_didIteratorError5) {
          throw _iteratorError5;
        }
      }
    }

    var _iteratorNormalCompletion6 = true;
    var _didIteratorError6 = false;
    var _iteratorError6 = undefined;

    try {
      for (var _iterator6 = (0, _getIterator3.default)(MixIns), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
        var _MixIn2 = _step6.value;
        var _iteratorNormalCompletion7 = true;
        var _didIteratorError7 = false;
        var _iteratorError7 = undefined;

        try {
          for (var _iterator7 = (0, _getIterator3.default)(_ownKeys(_MixIn2)), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
            var _prop3 = _step7.value;

            if (_isIgnoredStaticProperty(_prop3)) {
              continue;
            }

            if (_prop3 in descriptors) {
              throw new Error(`${name}.${_prop3} is already defined`);
            }

            descriptors[_prop3] = getOwnPropertyDescriptor(_MixIn2, _prop3);
          }
        } catch (err) {
          _didIteratorError7 = true;
          _iteratorError7 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion7 && _iterator7.return) {
              _iterator7.return();
            }
          } finally {
            if (_didIteratorError7) {
              throw _iteratorError7;
            }
          }
        }
      }
    } catch (err) {
      _didIteratorError6 = true;
      _iteratorError6 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion6 && _iterator6.return) {
          _iterator6.return();
        }
      } finally {
        if (_didIteratorError6) {
          throw _iteratorError6;
        }
      }
    }

    defineProperties(Decorator, descriptors);

    return Decorator;
  };
};
//# sourceMappingURL=decorators.js.map
