'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _startsWith2 = require('lodash/startsWith');

var _startsWith3 = _interopRequireDefault(_startsWith2);

var _isString2 = require('lodash/isString');

var _isString3 = _interopRequireDefault(_isString2);

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _isRedirect = require('is-redirect');

var _isRedirect2 = _interopRequireDefault(_isRedirect);

var _promiseToolbox = require('promise-toolbox');

var _http = require('http');

var _https = require('https');

var _querystring = require('querystring');

var _url = require('url');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// -------------------------------------------------------------------

var raw = function raw(opts) {
  var req = void 0;

  var pResponse = new _promise2.default(function (resolve, reject) {
    var body = opts.body,
        _opts$headers = opts.headers;
    _opts$headers = _opts$headers === undefined ? {} : _opts$headers;
    var headers = (0, _objectWithoutProperties3.default)(_opts$headers, []),
        protocol = opts.protocol,
        query = opts.query,
        rest = (0, _objectWithoutProperties3.default)(opts, ['body', 'headers', 'protocol', 'query']);


    if (headers['content-length'] == null && body != null) {
      var tmp = void 0;
      if ((0, _isString3.default)(body)) {
        headers['content-length'] = Buffer.byteLength(body);
      } else if ((tmp = body.headers) && (tmp = tmp['content-length']) != null || (tmp = body.length) != null) {
        headers['content-length'] = tmp;
      }
    }

    if (query) {
      rest.path = `${rest.pathname || rest.path || '/'}?${(0, _isString3.default)(query) ? query : (0, _querystring.stringify)(query)}`;
    }

    // Some headers can be explicitly removed by setting them to null.
    var headersToRemove = [];
    for (var header in headers) {
      if (headers[header] === null) {
        delete headers[header];
        headersToRemove.push(header);
      }
    }

    var secure = protocol && (0, _startsWith3.default)(protocol.toLowerCase(), 'https');
    var requestFn = void 0;
    if (secure) {
      requestFn = _https.request;
    } else {
      requestFn = _http.request;
      delete rest.rejectUnauthorized;
    }

    req = requestFn((0, _extends3.default)({}, rest, {
      headers
    }));

    for (var i = 0, length = headersToRemove.length; i < length; ++i) {
      req.removeHeader(headersToRemove[i]);
    }

    if (body) {
      if (typeof body.pipe === 'function') {
        body.pipe(req);
      } else {
        req.end(body);
      }
    } else {
      req.end();
    }
    req.on('error', reject);
    req.once('response', resolve);
  }).then(function (response) {
    response.cancel = function () {
      req.abort();
    };
    response.readAll = function () {
      return (0, _utils.streamToBuffer)(response);
    };

    var length = response.headers['content-length'];
    if (length) {
      response.length = length;
    }

    var code = response.statusCode;
    var location = response.headers.location;

    if ((0, _isRedirect2.default)(code) && location) {
      (0, _assign3.default)(opts, (0, _url.parse)((0, _url.resolve)((0, _url.format)(opts), location)));
      return raw(opts);
    }
    if (code < 200 || code >= 300) {
      var error = new Error(response.statusMessage);
      error.code = code;
      Object.defineProperty(error, 'response', {
        configurable: true,
        value: response,
        writable: true
      });

      throw error;
    }

    return response;
  });
  pResponse.request = req;

  return pResponse;
};

var httpRequestPlus = function httpRequestPlus($cancelToken) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  var opts = {};
  for (var i = 0, length = args.length; i < length; ++i) {
    var arg = args[i];
    (0, _assign3.default)(opts, (0, _isString3.default)(arg) ? (0, _url.parse)(arg) : arg);
  }

  var pResponse = raw(opts);

  $cancelToken.promise.then(function () {
    var request = pResponse.request;

    request.emit('error', new Error('HTTP request canceled!'));
    request.abort();
  });
  pResponse.readAll = function () {
    return pResponse.then(function (response) {
      return response.readAll();
    });
  };

  return pResponse;
};
exports.default = (0, _promiseToolbox.cancellable)(httpRequestPlus);
//# sourceMappingURL=http-request.js.map
