'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

// ===================================================================

var printLogs = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(db, args) {
    var stream, fields;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            stream = (0, _highland2.default)(db.createReadStream({ reverse: true }));


            if (args.since) {
              stream = stream.filter(function (_ref2) {
                var value = _ref2.value;
                return value.time >= args.since;
              });
            }

            if (args.until) {
              stream = stream.filter(function (_ref3) {
                var value = _ref3.value;
                return value.time <= args.until;
              });
            }

            fields = (0, _keys2.default)(args.matchers);


            if (fields.length > 0) {
              stream = stream.filter(function (_ref4) {
                var value = _ref4.value;
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                  for (var _iterator = (0, _getIterator3.default)(fields), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var field = _step.value;

                    var fieldValue = (0, _get2.default)(value, field);
                    if (fieldValue === undefined || !args.matchers[field](fieldValue)) {
                      return false;
                    }
                  }
                } catch (err) {
                  _didIteratorError = true;
                  _iteratorError = err;
                } finally {
                  try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                      _iterator.return();
                    }
                  } finally {
                    if (_didIteratorError) {
                      throw _iteratorError;
                    }
                  }
                }

                return true;
              });
            }

            stream = stream.take(args.limit);

            if (args.json) {
              stream = (0, _highland2.default)(stream.pipe(_ndjson2.default.serialize())).each(function (value) {
                process.stdout.write(value);
              });
            } else {
              stream = stream.each(function (value) {
                console.log(_util2.default.inspect(value, { depth: null }));
              });
            }

            return _context.abrupt('return', new _promise2.default(function (resolve) {
              stream.done(resolve);
            }));

          case 8:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function printLogs(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

// ===================================================================

var _appConf = require('app-conf');

var _appConf2 = _interopRequireDefault(_appConf);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _highland = require('highland');

var _highland2 = _interopRequireDefault(_highland);

var _levelParty = require('level-party');

var _levelParty2 = _interopRequireDefault(_levelParty);

var _ndjson = require('ndjson');

var _ndjson2 = _interopRequireDefault(_ndjson);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _levelSublevel = require('level-sublevel');

var _levelSublevel2 = _interopRequireDefault(_levelSublevel);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _leveldown = require('leveldown');

var _utils = require('./utils');

var _globMatcher = require('./glob-matcher');

var _globMatcher2 = _interopRequireDefault(_globMatcher);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function helper() {
  console.error(`
xo-server-logs --help, -h

    Display this help message.

xo-server-logs [--json] [--limit=<limit>] [--since=<date>] [--until=<date>] [<pattern>...]

    Prints the logs.

    --json
      Display the results as new line delimited JSON for consumption
      by another program.

    --limit=<limit>, -n <limit>
      Limit the number of results to be displayed (default 100)

    --since=<date>, --until=<date>
      Start showing entries on or newer than the specified date, or on
      or older than the specified date.

      <date> should use the format \`YYYY-MM-DD\`.

    <pattern>
      Patterns can be used to filter the entries.

      Patterns have the following format \`<field>=<value>\`/\`<field>\`.

xo-server-logs --repair

    Repair/compact the database.

    This is an advanced operation and should be used only when necessary and offline (xo-server should be stopped).
`);
}

// ===================================================================

function getArgs() {
  var stringArgs = ['since', 'until', 'limit'];
  var args = (0, _minimist2.default)(process.argv.slice(2), {
    string: stringArgs,
    boolean: ['help', 'json', 'repair'],
    default: {
      limit: 100,
      json: false,
      help: false
    },
    alias: {
      limit: 'n',
      help: 'h'
    }
  });

  var patterns = {};

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = (0, _getIterator3.default)(args._), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var value = _step2.value;

      value = String(value);

      var i = value.indexOf('=');

      if (i !== -1) {
        var _field = value.slice(0, i);
        var pattern = value.slice(i + 1);

        patterns[pattern] ? patterns[_field].push(pattern) : patterns[_field] = [pattern];
      } else if (!patterns[value]) {
        patterns[value] = null;
      }
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  var trueFunction = function trueFunction() {
    return true;
  };
  args.matchers = {};

  for (var field in patterns) {
    var values = patterns[field];
    args.matchers[field] = values === null ? trueFunction : (0, _globMatcher2.default)(values);
  }

  // Warning: minimist makes one array of values if the same option is used many times.
  // (But only for strings args, not boolean)
  (0, _utils.forEach)(stringArgs, function (arg) {
    if (args[arg] instanceof Array) {
      throw new Error(`error: too many values for ${arg} argument`);
    }
  });['since', 'until'].forEach(function (arg) {
    if (args[arg] !== undefined) {
      args[arg] = Date.parse(args[arg]);

      if (isNaN(args[arg])) {
        throw new Error(`error: bad ${arg} timestamp format`);
      }
    }
  });

  if (isNaN(args.limit = +args.limit)) {
    throw new Error('error: limit is not a valid number');
  }

  return args;
}

// ===================================================================

exports.default = function () {
  var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
    var args, config, db;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            args = getArgs();

            if (!args.help) {
              _context2.next = 4;
              break;
            }

            helper();
            return _context2.abrupt('return');

          case 4:
            _context2.next = 6;
            return _appConf2.default.load('xo-server', {
              ignoreUnknownFormats: true
            });

          case 6:
            config = _context2.sent;

            if (!args.repair) {
              _context2.next = 11;
              break;
            }

            _context2.next = 10;
            return new _promise2.default(function (resolve, reject) {
              (0, _leveldown.repair)(`${config.datadir}/leveldb`, function (error) {
                if (error) {
                  reject(error);
                } else {
                  resolve();
                }
              });
            });

          case 10:
            return _context2.abrupt('return');

          case 11:
            db = (0, _levelSublevel2.default)((0, _levelParty2.default)(`${config.datadir}/leveldb`, { valueEncoding: 'json' })).sublevel('logs');
            return _context2.abrupt('return', printLogs(db, args));

          case 13:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  function main() {
    return _ref5.apply(this, arguments);
  }

  return main;
}();
//# sourceMappingURL=logs-cli.js.map
