'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.crossProduct = exports.mergeObjects = exports.combine = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _assign = require('lodash/assign');

var _assign2 = _interopRequireDefault(_assign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _combine = function _combine(vectors, n, cb) {
  if (!n) {
    return;
  }

  var nLast = n - 1;

  var vector = vectors[nLast];
  var m = vector.length;
  if (n === 1) {
    for (var i = 0; i < m; ++i) {
      cb([vector[i]]); // eslint-disable-line standard/no-callback-literal
    }
    return;
  }

  var _loop = function _loop(_i) {
    var value = vector[_i];

    _combine(vectors, nLast, function (vector) {
      vector.push(value);
      cb(vector);
    });
  };

  for (var _i = 0; _i < m; ++_i) {
    _loop(_i);
  }
};

// Compute all combinations from vectors.
//
// Ex: combine([[2, 3], [5, 7]])
// => [ [ 2, 5 ], [ 3, 5 ], [ 2, 7 ], [ 3, 7 ] ]
var combine = exports.combine = function combine(vectors) {
  return function (cb) {
    return _combine(vectors, vectors.length, cb);
  };
};

// Merge the properties of an objects set in one object.
//
// Ex: mergeObjects([ { a: 1 }, { b: 2 } ]) => { a: 1, b: 2 }
var mergeObjects = exports.mergeObjects = function mergeObjects(objects) {
  return _assign2.default.apply(undefined, [{}].concat((0, _toConsumableArray3.default)(objects)));
};

// Compute a cross product between vectors.
//
// Ex: crossProduct([ [ { a: 2 }, { b: 3 } ], [ { c: 5 }, { d: 7 } ] ] )
// => [ { a: 2, c: 5 }, { b: 3, c: 5 }, { a: 2, d: 7 }, { b: 3, d: 7 } ]
var crossProduct = exports.crossProduct = function crossProduct(vectors) {
  var mergeFn = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : mergeObjects;
  return function (cb) {
    return combine(vectors)(function (vector) {
      cb(mergeFn(vector));
    });
  };
};
//# sourceMappingURL=math.js.map
