'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = proxyConsole;

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _partialStream = require('partial-stream');

var _partialStream2 = _interopRequireDefault(_partialStream);

var _tls = require('tls');

var _url = require('url');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('xo:proxy-console');

function proxyConsole(ws, vmConsole, sessionId) {
  var url = (0, _url.parse)(vmConsole.location);

  var closed = false;

  var socket = (0, _tls.connect)({
    host: url.host,
    port: url.port || 443,
    rejectUnauthorized: false
  }, function () {
    // Write headers.
    socket.write([`CONNECT ${url.path} HTTP/1.0`, `Host: ${url.hostname}`, `Cookie: session_id=${sessionId}`, '', ''].join('\r\n'));

    var onSend = function onSend(error) {
      if (error) {
        debug('error sending to the XO client: %s', error.stack || error.message || error);
      }
    };

    socket.pipe((0, _partialStream2.default)('\r\n\r\n', function (headers) {
      // TODO: check status code 200.
      debug('connected');
    })).on('data', function (data) {
      if (!closed) {
        // Encode to base 64.
        ws.send(data.toString('base64'), onSend);
      }
    }).on('end', function () {
      if (!closed) {
        closed = true;
        debug('disconnected from the console');
      }

      ws.close();
    });

    ws.on('error', function (error) {
      closed = true;
      debug('error from the XO client: %s', error.stack || error.message || error);

      socket.close();
    }).on('message', function (data) {
      if (!closed) {
        // Decode from base 64.
        socket.write(Buffer.from(data, 'base64'));
      }
    }).on('close', function () {
      if (!closed) {
        closed = true;
        debug('disconnected from the XO client');
      }

      socket.end();
    });
  }).on('error', function (error) {
    closed = true;
    debug('error from the console: %s', error.stack || error.message || error);

    ws.close();
  });
}
//# sourceMappingURL=proxy-console.js.map
