'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.FaultyGranularity = exports.UnknownLegendFormat = exports.XapiStatsError = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var getServerTimestamp = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(xapi, host) {
    var serverLocalTime;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return xapi.call('host.get_servertime', host.$ref);

          case 2:
            serverLocalTime = _context.sent;
            return _context.abrupt('return', Math.floor((0, _xapi.parseDateTime)(serverLocalTime).getTime() / 1000));

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getServerTimestamp(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

// -------------------------------------------------------------------
// Stats
// -------------------------------------------------------------------

var _endsWith = require('lodash/endsWith');

var _endsWith2 = _interopRequireDefault(_endsWith);

var _json = require('json5');

var _json2 = _interopRequireDefault(_json);

var _makeError = require('make-error');

var _httpRequest = require('./http-request');

var _httpRequest2 = _interopRequireDefault(_httpRequest);

var _xapi = require('./xapi');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var RRD_STEP_SECONDS = 5;
var RRD_STEP_MINUTES = 60;
var RRD_STEP_HOURS = 3600;
var RRD_STEP_DAYS = 86400;

var RRD_STEP_FROM_STRING = {
  'seconds': RRD_STEP_SECONDS,
  'minutes': RRD_STEP_MINUTES,
  'hours': RRD_STEP_HOURS,
  'days': RRD_STEP_DAYS
};

var RRD_POINTS_PER_STEP = {
  [RRD_STEP_SECONDS]: 120,
  [RRD_STEP_MINUTES]: 120,
  [RRD_STEP_HOURS]: 168,
  [RRD_STEP_DAYS]: 366
};

var XapiStatsError = exports.XapiStatsError = function (_BaseError) {
  (0, _inherits3.default)(XapiStatsError, _BaseError);

  function XapiStatsError() {
    (0, _classCallCheck3.default)(this, XapiStatsError);
    return (0, _possibleConstructorReturn3.default)(this, (XapiStatsError.__proto__ || (0, _getPrototypeOf2.default)(XapiStatsError)).apply(this, arguments));
  }

  return XapiStatsError;
}(_makeError.BaseError);

var UnknownLegendFormat = exports.UnknownLegendFormat = function (_XapiStatsError) {
  (0, _inherits3.default)(UnknownLegendFormat, _XapiStatsError);

  function UnknownLegendFormat(line) {
    (0, _classCallCheck3.default)(this, UnknownLegendFormat);
    return (0, _possibleConstructorReturn3.default)(this, (UnknownLegendFormat.__proto__ || (0, _getPrototypeOf2.default)(UnknownLegendFormat)).call(this, 'Unknown legend line: ' + line));
  }

  return UnknownLegendFormat;
}(XapiStatsError);

var FaultyGranularity = exports.FaultyGranularity = function (_XapiStatsError2) {
  (0, _inherits3.default)(FaultyGranularity, _XapiStatsError2);

  function FaultyGranularity() {
    (0, _classCallCheck3.default)(this, FaultyGranularity);
    return (0, _possibleConstructorReturn3.default)(this, (FaultyGranularity.__proto__ || (0, _getPrototypeOf2.default)(FaultyGranularity)).apply(this, arguments));
  }

  return FaultyGranularity;
}(XapiStatsError);

// -------------------------------------------------------------------
// Utils
// -------------------------------------------------------------------

function makeUrl(hostname, sessionId, timestamp) {
  return `https://${hostname}/rrd_updates?session_id=${sessionId}&start=${timestamp}&cf=AVERAGE&host=true&json=true`;
}

// Return current local timestamp in seconds
function getCurrentTimestamp() {
  return Date.now() / 1000;
}

function convertNanToNull(value) {
  return isNaN(value) ? null : value;
}

function getNewHostStats() {
  return {
    cpus: [],
    pifs: {
      rx: [],
      tx: []
    },
    load: [],
    memory: [],
    memoryFree: [],
    memoryUsed: []
  };
}

function getNewVmStats() {
  return {
    cpus: [],
    vifs: {
      rx: [],
      tx: []
    },
    xvds: {
      r: {},
      w: {}
    },
    memory: [],
    memoryFree: [],
    memoryUsed: []
  };
}

// -------------------------------------------------------------------
// Stats legends
// -------------------------------------------------------------------

function getNewHostLegends() {
  return {
    cpus: [],
    pifs: {
      rx: [],
      tx: []
    },
    load: null,
    memoryFree: null,
    memory: null
  };
}

function getNewVmLegends() {
  return {
    cpus: [],
    vifs: {
      rx: [],
      tx: []
    },
    xvds: {
      r: [],
      w: []
    },
    memoryFree: null,
    memory: null
  };
}

// Compute one legend line for one host
function parseOneHostLegend(hostLegend, type, index) {
  var resReg = void 0;

  if ((resReg = /^cpu([0-9]+)$/.exec(type)) !== null) {
    hostLegend.cpus[resReg[1]] = index;
  } else if ((resReg = /^pif_eth([0-9]+)_(rx|tx)$/.exec(type)) !== null) {
    if (resReg[2] === 'rx') {
      hostLegend.pifs.rx[resReg[1]] = index;
    } else {
      hostLegend.pifs.tx[resReg[1]] = index;
    }
  } else if (type === 'loadavg') {
    hostLegend.load = index;
  } else if (type === 'memory_free_kib') {
    hostLegend.memoryFree = index;
  } else if (type === 'memory_total_kib') {
    hostLegend.memory = index;
  }
}

// Compute one legend line for one vm
function parseOneVmLegend(vmLegend, type, index) {
  var resReg = void 0;

  if ((resReg = /^cpu([0-9]+)$/.exec(type)) !== null) {
    vmLegend.cpus[resReg[1]] = index;
  } else if ((resReg = /^vif_([0-9]+)_(rx|tx)$/.exec(type)) !== null) {
    if (resReg[2] === 'rx') {
      vmLegend.vifs.rx[resReg[1]] = index;
    } else {
      vmLegend.vifs.tx[resReg[1]] = index;
    }
  } else if (resReg = /^vbd_xvd(.)_(read|write)$/.exec(type)) {
    if (resReg[2] === 'read') {
      vmLegend.xvds.r[resReg[1]] = index;
    } else {
      vmLegend.xvds.w[resReg[1]] = index;
    }
  } else if (type === 'memory_internal_free') {
    vmLegend.memoryFree = index;
  } else if ((0, _endsWith2.default)(type, 'memory')) {
    vmLegend.memory = index;
  }
}

// Compute Stats Legends for host and vms from RRD update
function parseLegends(json) {
  var hostLegends = getNewHostLegends();
  var vmsLegends = {};

  json.meta.legend.forEach(function (value, index) {
    var parsedLine = /^AVERAGE:(host|vm):(.+):(.+)$/.exec(value);

    if (parsedLine === null) {
      throw new UnknownLegendFormat(value);
    }

    var _parsedLine = (0, _slicedToArray3.default)(parsedLine, 5),
        name = _parsedLine[1],
        uuid = _parsedLine[2],
        type = _parsedLine[3];

    if (name !== 'vm') {
      parseOneHostLegend(hostLegends, type, index);
    } else {
      if (vmsLegends[uuid] === undefined) {
        vmsLegends[uuid] = getNewVmLegends();
      }

      parseOneVmLegend(vmsLegends[uuid], type, index);
    }
  });

  return [hostLegends, vmsLegends];
}

var XapiStats = function () {
  function XapiStats() {
    (0, _classCallCheck3.default)(this, XapiStats);

    this._vms = {};
    this._hosts = {};
  }

  // -------------------------------------------------------------------
  // Remove stats (Helper)
  // -------------------------------------------------------------------

  (0, _createClass3.default)(XapiStats, [{
    key: '_removeOlderStats',
    value: function _removeOlderStats(source, dest, pointsPerStep) {
      for (var key in source) {
        if (key === 'cpus') {
          for (var cpuIndex in source.cpus) {
            dest.cpus[cpuIndex].splice(0, dest.cpus[cpuIndex].length - pointsPerStep);
          }

          // If the number of cpus has been decreased, remove !
          var offset = void 0;

          if ((offset = dest.cpus.length - source.cpus.length) > 0) {
            dest.cpus.splice(-offset);
          }
        } else if ((0, _endsWith2.default)(key, 'ifs')) {
          // For each pif or vif
          for (var ifType in source[key]) {
            for (var pifIndex in source[key][ifType]) {
              dest[key][ifType][pifIndex].splice(0, dest[key][ifType][pifIndex].length - pointsPerStep);
            }

            // If the number of pifs has been decreased, remove !
            var _offset = void 0;

            if ((_offset = dest[key][ifType].length - source[key][ifType].length) > 0) {
              dest[key][ifType].splice(-_offset);
            }
          }
        } else if (key === 'xvds') {
          for (var xvdType in source.xvds) {
            for (var xvdLetter in source.xvds[xvdType]) {
              dest.xvds[xvdType][xvdLetter].splice(0, dest.xvds[xvdType][xvdLetter].length - pointsPerStep);
            }

            // If the number of xvds has been decreased, remove !
            // FIXME
          }
        } else if (key === 'load') {
          dest.load.splice(0, dest[key].length - pointsPerStep);
        } else if (key === 'memory') {
          // Load, memory, memoryFree, memoryUsed
          var length = dest.memory.length - pointsPerStep;
          dest.memory.splice(0, length);
          dest.memoryFree.splice(0, length);
          dest.memoryUsed.splice(0, length);
        }
      }
    }

    // -------------------------------------------------------------------
    // HOST: Computation and stats update
    // -------------------------------------------------------------------

    // Compute one stats row for one host

  }, {
    key: '_parseRowHostStats',
    value: function _parseRowHostStats(hostLegends, hostStats, values) {
      // Cpus
      hostLegends.cpus.forEach(function (cpuIndex, index) {
        if (hostStats.cpus[index] === undefined) {
          hostStats.cpus[index] = [];
        }

        hostStats.cpus[index].push(values[cpuIndex] * 100);
      });

      // Pifs

      var _loop = function _loop(pifType) {
        hostLegends.pifs[pifType].forEach(function (pifIndex, index) {
          if (hostStats.pifs[pifType][index] === undefined) {
            hostStats.pifs[pifType][index] = [];
          }

          hostStats.pifs[pifType][index].push(convertNanToNull(values[pifIndex]));
        });
      };

      for (var pifType in hostLegends.pifs) {
        _loop(pifType);
      }

      // Load
      hostStats.load.push(convertNanToNull(values[hostLegends.load]));

      // Memory.
      // WARNING! memory/memoryFree are in kB.
      var memory = values[hostLegends.memory] * 1024;
      var memoryFree = values[hostLegends.memoryFree] * 1024;

      hostStats.memory.push(memory);

      if (hostLegends.memoryFree !== undefined) {
        hostStats.memoryFree.push(memoryFree);
        hostStats.memoryUsed.push(memory - memoryFree);
      }
    }

    // Compute stats for host from RRD update

  }, {
    key: '_parseHostStats',
    value: function _parseHostStats(json, hostname, hostLegends, step) {
      var host = this._hosts[hostname][step];

      if (host.stats === undefined) {
        host.stats = getNewHostStats();
      }

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(json.data), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var row = _step.value;

          this._parseRowHostStats(hostLegends, host.stats, row.values);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }

    // -------------------------------------------------------------------
    // VM: Computation and stats update
    // -------------------------------------------------------------------

    // Compute stats for vms from RRD update

  }, {
    key: '_parseRowVmStats',
    value: function _parseRowVmStats(vmLegends, vmStats, values) {
      // Cpus
      vmLegends.cpus.forEach(function (cpuIndex, index) {
        if (vmStats.cpus[index] === undefined) {
          vmStats.cpus[index] = [];
        }

        vmStats.cpus[index].push(values[cpuIndex] * 100);
      });

      // Vifs

      var _loop2 = function _loop2(vifType) {
        vmLegends.vifs[vifType].forEach(function (vifIndex, index) {
          if (vmStats.vifs[vifType][index] === undefined) {
            vmStats.vifs[vifType][index] = [];
          }

          vmStats.vifs[vifType][index].push(convertNanToNull(values[vifIndex]));
        });
      };

      for (var vifType in vmLegends.vifs) {
        _loop2(vifType);
      }

      // Xvds
      for (var xvdType in vmLegends.xvds) {
        for (var index in vmLegends.xvds[xvdType]) {
          if (vmStats.xvds[xvdType][index] === undefined) {
            vmStats.xvds[xvdType][index] = [];
          }

          vmStats.xvds[xvdType][index].push(convertNanToNull(values[vmLegends.xvds[xvdType][index]]));
        }
      }

      // Memory
      // WARNING! memoryFree is in Kb not in b, memory is in b
      var memory = values[vmLegends.memory];
      var memoryFree = values[vmLegends.memoryFree] * 1024;

      vmStats.memory.push(memory);

      if (vmLegends.memoryFree !== undefined) {
        vmStats.memoryFree.push(memoryFree);
        vmStats.memoryUsed.push(memory - memoryFree);
      }
    }

    // Compute stats for vms

  }, {
    key: '_parseVmsStats',
    value: function _parseVmsStats(json, hostname, vmsLegends, step) {
      if (this._vms[hostname][step] === undefined) {
        this._vms[hostname][step] = {};
      }

      var vms = this._vms[hostname][step];

      for (var uuid in vmsLegends) {
        if (vms[uuid] === undefined) {
          vms[uuid] = getNewVmStats();
        }
      }

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator3.default)(json.data), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var row = _step2.value;

          for (var _uuid in vmsLegends) {
            this._parseRowVmStats(vmsLegends[_uuid], vms[_uuid], row.values);
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }

    // -------------------------------------------------------------------
    // -------------------------------------------------------------------

    // Execute one http request on a XenServer for get stats
    // Return stats (Json format) or throws got exception

  }, {
    key: '_getJson',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(url) {
        var body;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return (0, _httpRequest2.default)(url, { rejectUnauthorized: false }).readAll();

              case 2:
                body = _context2.sent;
                return _context2.abrupt('return', _json2.default.parse(body));

              case 4:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function _getJson(_x3) {
        return _ref2.apply(this, arguments);
      }

      return _getJson;
    }()
  }, {
    key: '_getLastTimestamp',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(xapi, host, step) {
        var serverTimeStamp;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!(this._hosts[host.address][step] === undefined)) {
                  _context3.next = 5;
                  break;
                }

                _context3.next = 3;
                return getServerTimestamp(xapi, host);

              case 3:
                serverTimeStamp = _context3.sent;
                return _context3.abrupt('return', serverTimeStamp - step * RRD_POINTS_PER_STEP[step] + step);

              case 5:
                return _context3.abrupt('return', this._hosts[host.address][step].endTimestamp);

              case 6:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function _getLastTimestamp(_x4, _x5, _x6) {
        return _ref3.apply(this, arguments);
      }

      return _getLastTimestamp;
    }()
  }, {
    key: '_getPoints',
    value: function _getPoints(hostname, step, vmId) {
      var hostStats = this._hosts[hostname][step];

      // Return host points
      if (vmId === undefined) {
        return (0, _extends3.default)({
          interval: step
        }, hostStats);
      }

      var vmsStats = this._vms[hostname][step];

      // Return vm points
      return {
        interval: step,
        endTimestamp: hostStats.endTimestamp,
        stats: vmsStats && vmsStats[vmId] || getNewVmStats()
      };
    }
  }, {
    key: '_getAndUpdatePoints',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(xapi, host, vmId, granularity) {
        var step, hostname, timestamp, json, serverTimestamp, startTimestamp, parseOffset, _parseLegends, _parseLegends2, hostLegends, vmsLegends, uuid;

        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                // Get granularity to use
                step = granularity === undefined || granularity === 0 ? RRD_STEP_SECONDS : RRD_STEP_FROM_STRING[granularity];

                if (!(step === undefined)) {
                  _context4.next = 3;
                  break;
                }

                throw new FaultyGranularity(`Unknown granularity: '${granularity}'. Use 'seconds', 'minutes', 'hours', or 'days'.`);

              case 3:

                // Limit the number of http requests
                hostname = host.address;


                if (this._hosts[hostname] === undefined) {
                  this._hosts[hostname] = {};
                  this._vms[hostname] = {};
                }

                if (!(this._hosts[hostname][step] !== undefined && this._hosts[hostname][step].localTimestamp + step > getCurrentTimestamp())) {
                  _context4.next = 7;
                  break;
                }

                return _context4.abrupt('return', this._getPoints(hostname, step, vmId));

              case 7:
                _context4.next = 9;
                return this._getLastTimestamp(xapi, host, step);

              case 9:
                timestamp = _context4.sent;
                _context4.next = 12;
                return this._getJson(makeUrl(hostname, xapi.sessionId, timestamp));

              case 12:
                json = _context4.sent;

                if (!(json.meta.step !== step)) {
                  _context4.next = 23;
                  break;
                }

                console.log(`RRD call: Expected step: ${step}, received step: ${json.meta.step}. Retry with other timestamp`);
                _context4.next = 17;
                return getServerTimestamp(xapi, host);

              case 17:
                serverTimestamp = _context4.sent;
                _context4.next = 20;
                return this._getJson(makeUrl(hostname, xapi.sessionId, serverTimestamp - step * (RRD_POINTS_PER_STEP[step] / 2) + step));

              case 20:
                json = _context4.sent;

                if (!(json.meta.step !== step)) {
                  _context4.next = 23;
                  break;
                }

                throw new FaultyGranularity(`Unable to get the true granularity: ${json.meta.step}`);

              case 23:

                // Make new backup slot if necessary
                if (this._hosts[hostname][step] === undefined) {
                  this._hosts[hostname][step] = {
                    endTimestamp: 0,
                    localTimestamp: 0
                  };
                }

                // It exists data
                if (json.data.length !== 0) {
                  // Warning: Sometimes, the json.xport.meta.start value does not match with the
                  // timestamp of the oldest data value
                  // So, we use the timestamp of the oldest data value !
                  startTimestamp = json.data[json.meta.rows - 1].t;

                  // Remove useless data and reorder
                  // Note: Older values are at end of json.data.row

                  parseOffset = (this._hosts[hostname][step].endTimestamp - startTimestamp + step) / step;


                  json.data.splice(json.data.length - parseOffset);
                  json.data.reverse();

                  // It exists useful data
                  if (json.data.length > 0) {
                    _parseLegends = parseLegends(json), _parseLegends2 = (0, _slicedToArray3.default)(_parseLegends, 2), hostLegends = _parseLegends2[0], vmsLegends = _parseLegends2[1];

                    // Compute and update host/vms stats

                    this._parseVmsStats(json, hostname, vmsLegends, step);
                    this._parseHostStats(json, hostname, hostLegends, step);

                    // Remove older stats
                    this._removeOlderStats(hostLegends, this._hosts[hostname][step].stats, RRD_POINTS_PER_STEP[step]);

                    for (uuid in vmsLegends) {
                      this._removeOlderStats(vmsLegends[uuid], this._vms[hostname][step][uuid], RRD_POINTS_PER_STEP[step]);
                    }
                  }
                }

                // Update timestamp
                this._hosts[hostname][step].endTimestamp = json.meta.end;
                this._hosts[hostname][step].localTimestamp = getCurrentTimestamp();

                return _context4.abrupt('return', this._getPoints(hostname, step, vmId));

              case 28:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function _getAndUpdatePoints(_x7, _x8, _x9, _x10) {
        return _ref4.apply(this, arguments);
      }

      return _getAndUpdatePoints;
    }()

    // -------------------------------------------------------------------
    // -------------------------------------------------------------------

    // Warning: This functions returns one reference on internal data
    // So, data can be changed by a parallel call on this functions
    // It is forbidden to modify the returned data

    // Return host stats

  }, {
    key: 'getHostPoints',
    value: function () {
      var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5(xapi, hostId, granularity) {
        var host;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                host = xapi.getObject(hostId);
                return _context5.abrupt('return', this._getAndUpdatePoints(xapi, host, undefined, granularity));

              case 2:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function getHostPoints(_x11, _x12, _x13) {
        return _ref5.apply(this, arguments);
      }

      return getHostPoints;
    }()

    // Return vms stats

  }, {
    key: 'getVmPoints',
    value: function () {
      var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6(xapi, vmId, granularity) {
        var vm, host;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                vm = xapi.getObject(vmId);
                host = vm.$resident_on;

                if (host) {
                  _context6.next = 4;
                  break;
                }

                throw new Error(`VM ${vmId} is halted or host could not be found.`);

              case 4:
                return _context6.abrupt('return', this._getAndUpdatePoints(xapi, host, vm.uuid, granularity));

              case 5:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function getVmPoints(_x14, _x15, _x16) {
        return _ref6.apply(this, arguments);
      }

      return getVmPoints;
    }()
  }]);
  return XapiStats;
}();

exports.default = XapiStats;
//# sourceMappingURL=xapi-stats.js.map
