'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useUpdateSystem = exports.put = exports.NULL_REF = exports.makeEditObject = exports.isVmRunning = exports.isVmHvm = exports.isHostRunning = exports.parseDateTime = exports.formatDateTime = exports.getNamespaceForType = exports.extractOpaqueRef = exports.debug = exports.prepareXapiParam = exports.optional = exports.filterUndefineds = exports.asInteger = exports.asBoolean = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _camelCase = require('lodash/camelCase');

var _camelCase2 = _interopRequireDefault(_camelCase);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _isEqual = require('lodash/isEqual');

var _isEqual2 = _interopRequireDefault(_isEqual);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _pickBy = require('lodash/pickBy');

var _pickBy2 = _interopRequireDefault(_pickBy);

var _d3TimeFormat = require('d3-time-format');

var _semver = require('semver');

var _httpRequest = require('../http-request');

var _httpRequest2 = _interopRequireDefault(_httpRequest);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ===================================================================

var asBoolean = exports.asBoolean = function asBoolean(value) {
  return Boolean(value);
};

// const asFloat = value => {
//   value = String(value)
//   return value.indexOf('.') === -1
//     ? `${value}.0`
//     : value
// }

// import isFinite from 'lodash/isFinite'
var asInteger = exports.asInteger = function asInteger(value) {
  return String(value);
};

var filterUndefineds = exports.filterUndefineds = function filterUndefineds(obj) {
  return (0, _pickBy2.default)(obj, function (value) {
    return value !== undefined;
  });
};

var optional = exports.optional = function optional(value, fn) {
  return value == null ? undefined : fn ? fn(value) : value;
};

var prepareXapiParam = exports.prepareXapiParam = function prepareXapiParam(param) {
  // if (isFinite(param) && !isInteger(param)) {
  //   return asFloat(param)
  // }
  if ((0, _utils.isInteger)(param)) {
    return asInteger(param);
  }
  if ((0, _utils.isBoolean)(param)) {
    return asBoolean(param);
  }
  if ((0, _utils.isArray)(param)) {
    return (0, _utils.map)(param, prepareXapiParam);
  }
  if ((0, _isPlainObject2.default)(param)) {
    return (0, _utils.map)(filterUndefineds(param), prepareXapiParam);
  }

  return param;
};

// -------------------------------------------------------------------

var debug = exports.debug = (0, _debug2.default)('xo:xapi');

// -------------------------------------------------------------------

var OPAQUE_REF_RE = /OpaqueRef:[0-9a-z-]+/;
var extractOpaqueRef = exports.extractOpaqueRef = function extractOpaqueRef(str) {
  var matches = OPAQUE_REF_RE.exec(str);
  if (!matches) {
    throw new Error('no opaque ref found');
  }
  return matches[0];
};

// -------------------------------------------------------------------

var TYPE_TO_NAMESPACE = (0, _utils.createRawObject)();
(0, _utils.forEach)(['Bond', 'DR_task', 'GPU_group', 'PBD', 'PCI', 'PGPU', 'PIF', 'PIF_metrics', 'SM', 'SR', 'VBD', 'VBD_metrics', 'VDI', 'VGPU', 'VGPU_type', 'VIF', 'VLAN', 'VM', 'VM_appliance', 'VM_guest_metrics', 'VM_metrics', 'VMPP', 'VTPM'], function (namespace) {
  TYPE_TO_NAMESPACE[namespace.toLowerCase()] = namespace;
});

// Object types given by `xen-api` are always lowercase but the
// namespaces in the Xen API can have a different casing.
var getNamespaceForType = exports.getNamespaceForType = function getNamespaceForType(type) {
  return TYPE_TO_NAMESPACE[type] || type;
};

// -------------------------------------------------------------------

// Format a date (pseudo ISO 8601) from one XenServer get by
// xapi.call('host.get_servertime', host.$ref) for example
var formatDateTime = exports.formatDateTime = (0, _d3TimeFormat.utcFormat)('%Y%m%dT%H:%M:%SZ');

var parseDateTime = exports.parseDateTime = (0, _d3TimeFormat.utcParse)('%Y%m%dT%H:%M:%SZ');

// -------------------------------------------------------------------

var isHostRunning = exports.isHostRunning = function isHostRunning(host) {
  var $metrics = host.$metrics;


  return $metrics && $metrics.live;
};

// -------------------------------------------------------------------

var isVmHvm = exports.isVmHvm = function isVmHvm(vm) {
  return Boolean(vm.HVM_boot_policy);
};

var VM_RUNNING_POWER_STATES = {
  Running: true,
  Paused: true
};
var isVmRunning = exports.isVmRunning = function isVmRunning(vm) {
  return VM_RUNNING_POWER_STATES[vm.power_state];
};

// -------------------------------------------------------------------

var _DEFAULT_ADD_TO_LIMITS = function _DEFAULT_ADD_TO_LIMITS(next, current) {
  return next - current;
};

var makeEditObject = exports.makeEditObject = function makeEditObject(specs) {
  var normalizeGet = function normalizeGet(get, name) {
    if (get === true) {
      var prop = (0, _utils.camelToSnakeCase)(name);
      return function (object) {
        return object[prop];
      };
    }

    if ((0, _utils.isString)(get)) {
      return function (object) {
        return object[get];
      };
    }

    return get;
  };
  var normalizeSet = function normalizeSet(set, name) {
    if ((0, _utils.isFunction)(set)) {
      return set;
    }

    if (set === true) {
      var prop = (0, _utils.camelToSnakeCase)(name);
      return function (value) {
        return this._set(prop, value);
      };
    }

    if ((0, _utils.isString)(set)) {
      var index = set.indexOf('.');
      if (index === -1) {
        var _prop2 = (0, _utils.camelToSnakeCase)(set);
        return function (value) {
          return this._set(_prop2, value);
        };
      }

      var _map = set.slice(0, index);
      var _prop = set.slice(index + 1);

      return function (value, object) {
        return this._updateObjectMapProperty(object, _map, { [_prop]: value });
      };
    }

    if (!(0, _utils.isArray)(set)) {
      throw new Error('must be an array, a function or a string');
    }

    set = (0, _utils.mapToArray)(set, normalizeSet);

    var _set = set,
        length = _set.length;

    if (!length) {
      throw new Error('invalid setter');
    }

    if (length === 1) {
      return set[0];
    }

    return function (value, object) {
      var _this = this;

      return _promise2.default.all((0, _utils.mapToArray)(set, function (set) {
        return set.call(_this, value, object);
      }));
    };
  };

  var normalizeSpec = function normalizeSpec(spec, name) {
    if (spec === true) {
      spec = {
        get: true,
        set: true
      };
    }

    if (spec.addToLimits === true) {
      spec.addToLimits = _DEFAULT_ADD_TO_LIMITS;
    }
    if (!spec.limitName) {
      spec.limitName = name;
    }

    (0, _utils.forEach)(spec.constraints, function (constraint, constraintName) {
      if (!(0, _utils.isFunction)(constraint)) {
        throw new Error('constraint must be a function');
      }

      var constraintSpec = specs[constraintName];
      if (!constraintSpec.get) {
        throw new Error('constraint values must have a get');
      }
    });

    var _spec = spec,
        get = _spec.get;

    if (get) {
      spec.get = normalizeGet(get, name);
    } else if (spec.addToLimits) {
      throw new Error('addToLimits cannot be defined without get');
    }

    spec.set = normalizeSet(spec.set, name);

    return spec;
  };
  (0, _utils.forEach)(specs, function (spec, name) {
    (0, _utils.isString)(spec) || (specs[name] = normalizeSpec(spec, name));
  });

  // Resolves aliases and add camelCase and snake_case aliases.
  (0, _utils.forEach)(specs, function (spec, name) {
    if ((0, _utils.isString)(spec)) {
      do {
        spec = specs[spec];
      } while ((0, _utils.isString)(spec));
      specs[name] = spec;
    }

    var tmp = void 0;
    specs[tmp = (0, _camelCase2.default)(name)] || (specs[tmp] = spec);
    specs[tmp = (0, _utils.camelToSnakeCase)(name)] || (specs[tmp] = spec);
  });

  return function () {
    var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(id, values, checkLimits) {
      var _this2 = this;

      var limits, object, _objectRef, _setMethodPrefix, context, set, cbs;

      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              limits = checkLimits && {};
              object = this.getObject(id);
              _objectRef = object.$ref;
              _setMethodPrefix = `${getNamespaceForType(object.$type)}.set_`;

              // Context used to execute functions.

              context = {
                __proto__: this,
                _set: function _set(prop, value) {
                  return _this2.call(_setMethodPrefix + prop, _objectRef, prepareXapiParam(value));
                }
              };

              set = function set(value, name) {
                if (value === undefined) {
                  return;
                }

                var spec = specs[name];
                if (!spec) {
                  return;
                }

                var preprocess = spec.preprocess;

                if (preprocess) {
                  value = preprocess(value);
                }

                var get = spec.get;

                if (get) {
                  var current = get(object);
                  if ((0, _isEqual2.default)(value, current)) {
                    return;
                  }

                  var addToLimits = void 0;
                  if (limits && (addToLimits = spec.addToLimits)) {
                    limits[spec.limitName] = addToLimits(value, current);
                  }
                }

                var cb = function cb() {
                  return spec.set.call(context, value, object);
                };

                var constraints = spec.constraints;

                if (constraints) {
                  var _cbs = [];

                  (0, _utils.forEach)(constraints, function (constraint, constraintName) {
                    // Before setting a property to a new value, if the constraint check fails (e.g. memoryMin > memoryMax):
                    //    - if the user wants to set the constraint (ie constraintNewValue is defined):
                    //        constraint <-- constraintNewValue THEN property <-- value (e.g. memoryMax <-- 2048 THEN memoryMin <-- 1024)
                    //    - if the user DOES NOT want to set the constraint (ie constraintNewValue is NOT defined):
                    //        constraint <-- value THEN property <-- value (e.g. memoryMax <-- 1024 THEN memoryMin <-- 1024)
                    // FIXME: Some values combinations will lead to setting the same property twice, which is not perfect but works for now.
                    var constraintCurrentValue = specs[constraintName].get(object);
                    var constraintNewValue = values[constraintName];

                    if (!constraint(constraintCurrentValue, value)) {
                      var _cb = set(constraintNewValue == null ? value : constraintNewValue, constraintName);
                      if (_cb) {
                        _cbs.push(_cb);
                      }
                    }
                  });

                  if (_cbs.length) {
                    return function () {
                      return _promise2.default.all((0, _utils.mapToArray)(_cbs, function (cb) {
                        return cb();
                      })).then(cb);
                    };
                  }
                }

                return cb;
              };

              cbs = (0, _utils.mapFilter)(values, set);

              if (!checkLimits) {
                _context.next = 10;
                break;
              }

              _context.next = 10;
              return checkLimits(limits, object);

            case 10:
              return _context.abrupt('return', _promise2.default.all((0, _utils.mapToArray)(cbs, function (cb) {
                return cb();
              })).then(_utils.noop));

            case 11:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function _editObject_(_x, _x2, _x3) {
      return _ref.apply(this, arguments);
    }

    return _editObject_;
  }();
};

// ===================================================================

var NULL_REF = exports.NULL_REF = 'OpaqueRef:NULL';

// ===================================================================

// HTTP put, use an ugly hack if the length is not known because XAPI
// does not support chunk encoding.
var put = function put(stream, _ref2, task) {
  var _ref2$headers = _ref2.headers;
  _ref2$headers = _ref2$headers === undefined ? {} : _ref2$headers;
  var headers = (0, _objectWithoutProperties3.default)(_ref2$headers, []),
      opts = (0, _objectWithoutProperties3.default)(_ref2, ['headers']);

  var makeRequest = function makeRequest() {
    return (0, _httpRequest2.default)((0, _extends3.default)({}, opts, {
      body: stream,
      headers,
      method: 'put'
    }));
  };

  // Xen API does not support chunk encoding.
  if (stream.length == null) {
    headers['transfer-encoding'] = null;

    var promise = makeRequest();

    if (task) {
      // Some connections need the task to resolve (VDI import).
      _utils.pFinally.call(task, function () {
        promise.cancel();
      });
    } else {
      // Some tasks need the connection to close (VM import).
      promise.request.once('finish', function () {
        promise.cancel();
      });
    }

    return promise.readAll();
  }

  return makeRequest().readAll();
};

exports.put = put;
var useUpdateSystem = exports.useUpdateSystem = function useUpdateSystem(host) {
  // Match Xen Center's condition: https://github.com/xenserver/xenadmin/blob/f3a64fc54bbff239ca6f285406d9034f57537d64/XenModel/Utils/Helpers.cs#L420
  return (0, _semver.satisfies)(host.software_version.platform_version, '^2.1.1');
};
//# sourceMappingURL=utils.js.map
