'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _kindof = require('kindof');

var _kindof2 = _interopRequireDefault(_kindof);

var _ms = require('ms');

var _ms2 = _interopRequireDefault(_ms);

var _schemaInspector = require('schema-inspector');

var _schemaInspector2 = _interopRequireDefault(_schemaInspector);

var _api = require('../api');

var methods = _interopRequireWildcard(_api);

var _jsonRpcPeer = require('json-rpc-peer');

var _utils = require('../utils');

var _apiErrors = require('xo-common/api-errors');

var errors = _interopRequireWildcard(_apiErrors);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ===================================================================

var debug = (0, _debug2.default)('xo:api');

var PERMISSIONS = {
  none: 0,
  read: 1,
  write: 2,
  admin: 3
};

// TODO:
// - error when adding a server to a pool with incompatible version
// - error when halted VM migration failure is due to XS < 7
var XAPI_ERROR_TO_XO_ERROR = {
  EHOSTUNREACH: errors.serverUnreachable,
  HOST_OFFLINE: function HOST_OFFLINE(_ref, getId) {
    var _ref2 = (0, _slicedToArray3.default)(_ref, 1),
        host = _ref2[0];

    return errors.hostOffline({ host: getId(host) });
  },
  NO_HOSTS_AVAILABLE: errors.noHostsAvailable,
  NOT_SUPPORTED_DURING_UPGRADE: errors.notSupportedDuringUpgrade,
  OPERATION_BLOCKED: function OPERATION_BLOCKED(_ref3, getId) {
    var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
        ref = _ref4[0],
        code = _ref4[1];

    return errors.operationBlocked({ objectId: getId(ref), code });
  },
  PATCH_PRECHECK_FAILED_ISO_MOUNTED: function PATCH_PRECHECK_FAILED_ISO_MOUNTED(_ref5) {
    var _ref6 = (0, _slicedToArray3.default)(_ref5, 1),
        patch = _ref6[0];

    return errors.patchPrecheck({ errorType: 'isoMounted', patch });
  },
  PIF_VLAN_EXISTS: function PIF_VLAN_EXISTS(_ref7, getId) {
    var _ref8 = (0, _slicedToArray3.default)(_ref7, 1),
        pif = _ref8[0];

    return errors.objectAlreadyExists({ objectId: getId(pif), objectType: 'PIF' });
  },
  SESSION_AUTHENTICATION_FAILED: errors.authenticationFailed,
  VDI_IN_USE: function VDI_IN_USE(_ref9, getId) {
    var _ref10 = (0, _slicedToArray3.default)(_ref9, 2),
        vdi = _ref10[0],
        operation = _ref10[1];

    return errors.vdiInUse({ vdi: getId(vdi), operation });
  },
  VM_BAD_POWER_STATE: function VM_BAD_POWER_STATE(_ref11, getId) {
    var _ref12 = (0, _slicedToArray3.default)(_ref11, 3),
        vm = _ref12[0],
        expected = _ref12[1],
        actual = _ref12[2];

    return errors.vmBadPowerState({ vm: getId(vm), expected, actual });
  },
  VM_IS_TEMPLATE: errors.vmIsTemplate,
  VM_LACKS_FEATURE: function VM_LACKS_FEATURE(_ref13, getId) {
    var _ref14 = (0, _slicedToArray3.default)(_ref13, 1),
        vm = _ref14[0];

    return errors.vmLacksFeature({ vm: getId(vm) });
  },
  VM_LACKS_FEATURE_SHUTDOWN: function VM_LACKS_FEATURE_SHUTDOWN(_ref15, getId) {
    var _ref16 = (0, _slicedToArray3.default)(_ref15, 1),
        vm = _ref16[0];

    return errors.vmLacksFeature({ vm: getId(vm), feature: 'shutdown' });
  },
  VM_MISSING_PV_DRIVERS: function VM_MISSING_PV_DRIVERS(_ref17, getId) {
    var _ref18 = (0, _slicedToArray3.default)(_ref17, 1),
        vm = _ref18[0];

    return errors.vmMissingPvDrivers({ vm: getId(vm) });
  }
};

var hasPermission = function hasPermission(user, permission) {
  return PERMISSIONS[user.permission] >= PERMISSIONS[permission];
};

function checkParams(method, params) {
  var schema = method.params;
  if (!schema) {
    return;
  }

  var result = _schemaInspector2.default.validate({
    type: 'object',
    properties: schema
  }, params);

  if (!result.valid) {
    throw errors.invalidParameters(result.error);
  }
}

function checkPermission(method) {
  /* jshint validthis: true */

  var permission = method.permission;

  // No requirement.

  if (permission === undefined) {
    return;
  }

  var user = this.user;

  if (!user) {
    throw errors.unauthorized();
  }

  // The only requirement is login.
  if (!permission) {
    return;
  }

  if (!hasPermission(user, permission)) {
    throw errors.unauthorized();
  }
}

function resolveParams(method, params) {
  var _this = this;

  var resolve = method.resolve;
  if (!resolve) {
    return params;
  }

  var user = this.user;

  if (!user) {
    throw errors.unauthorized();
  }

  var userId = user.id;

  // Do not alter the original object.
  params = (0, _extends3.default)({}, params);

  var permissions = [];
  (0, _utils.forEach)(resolve, function (_ref19, key) {
    var _ref20 = (0, _slicedToArray3.default)(_ref19, 3),
        param = _ref20[0],
        types = _ref20[1],
        _ref20$ = _ref20[2],
        permission = _ref20$ === undefined ? 'administrate' : _ref20$;

    var id = params[param];
    if (id === undefined) {
      return;
    }

    var object = _this.getObject(id, types);

    // This parameter has been handled, remove it.
    delete params[param];

    // Register this new value.
    params[key] = object;

    // Permission default to 'administrate' but can be set to a falsy
    // value (except null or undefined which trigger the default
    // value) to simply do a resolve without checking any permissions.
    if (permission) {
      permissions.push([object.id, permission]);
    }
  });

  return this.hasPermissions(userId, permissions).then(function (success) {
    if (success) {
      return params;
    }

    throw errors.unauthorized();
  });
}

// ===================================================================

var Api = function () {
  function Api(xo) {
    var _this2 = this;

    (0, _classCallCheck3.default)(this, Api);

    this._logger = null;
    this._methods = (0, _utils.createRawObject)();
    this._xo = xo;

    this.addApiMethods(methods);
    xo.on('start', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return xo.getLogger('api');

            case 2:
              _this2._logger = _context.sent;

            case 3:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this2);
    })));
  }

  (0, _createClass3.default)(Api, [{
    key: 'addApiMethod',
    value: function addApiMethod(name, method) {
      var methods = this._methods;

      if (name in methods) {
        throw new Error(`API method ${name} already exists`);
      }

      methods[name] = method;

      var _remove = function remove() {
        delete methods[name];
        _remove = _utils.noop;
      };
      return function () {
        return _remove();
      };
    }
  }, {
    key: 'addApiMethods',
    value: function addApiMethods(methods) {
      var _this3 = this;

      var base = '';
      var removes = [];

      var addMethod = function addMethod(method, name) {
        name = base + name;

        if ((0, _utils.isFunction)(method)) {
          removes.push(_this3.addApiMethod(name, method));
          return;
        }

        var oldBase = base;
        base = name + '.';
        (0, _utils.forEach)(method, addMethod);
        base = oldBase;
      };

      try {
        (0, _utils.forEach)(methods, addMethod);
      } catch (error) {
        // Remove all added methods.
        (0, _utils.forEach)(removes, function (remove) {
          return remove();
        });

        // Forward the error
        throw error;
      }

      var _remove2 = function remove() {
        (0, _utils.forEach)(removes, function (remove) {
          return remove();
        });
        _remove2 = _utils.noop;
      };
      return _remove2;
    }
  }, {
    key: 'callApiMethod',
    value: function () {
      var _ref22 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(session, name, params) {
        var _this4 = this;

        var startTime, method, context, userId, userName, namespace, resolvedParams, result, data, message, stack, xoError;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                startTime = Date.now();
                method = this._methods[name];

                if (method) {
                  _context2.next = 4;
                  break;
                }

                throw new _jsonRpcPeer.MethodNotFound(name);

              case 4:

                // FIXME: it can cause issues if there any property assignments in
                // XO methods called from the API.
                context = (0, _create2.default)(this._xo, {
                  api: { // Used by system.*().
                    value: this
                  },
                  session: {
                    value: session
                  }
                });

                // Fetch and inject the current user.

                userId = session.get('user_id', undefined);
                _context2.t0 = userId;

                if (!_context2.t0) {
                  _context2.next = 11;
                  break;
                }

                _context2.next = 10;
                return this._xo.getUser(userId);

              case 10:
                _context2.t0 = _context2.sent;

              case 11:
                context.user = _context2.t0;
                userName = context.user ? context.user.email : '(unknown user)';
                _context2.prev = 13;
                _context2.next = 16;
                return checkPermission.call(context, method);

              case 16:

                // API methods are in a namespace.
                // Some methods use the namespace or an id parameter like:
                //
                // vm.detachPci vm=<string>
                // vm.ejectCd id=<string>
                //
                // The goal here is to standardize the calls by always providing
                // an id parameter when possible to simplify calls to the API.
                if (params && params.id === undefined) {
                  namespace = name.slice(0, name.indexOf('.'));

                  params.id = params[namespace];
                }

                checkParams.call(context, method, params);

                _context2.next = 20;
                return resolveParams.call(context, method, params);

              case 20:
                resolvedParams = _context2.sent;
                _context2.next = 23;
                return method.call(context, resolvedParams);

              case 23:
                result = _context2.sent;


                // If nothing was returned, consider this operation a success
                // and return true.
                if (result === undefined) {
                  result = true;
                }

                debug('%s | %s(...) [%s] ==> %s', userName, name, (0, _ms2.default)(Date.now() - startTime), (0, _kindof2.default)(result));

                return _context2.abrupt('return', result);

              case 29:
                _context2.prev = 29;
                _context2.t1 = _context2['catch'](13);
                data = {
                  userId,
                  method: name,
                  params,
                  duration: Date.now() - startTime,
                  error: (0, _utils.serializeError)(_context2.t1)
                };
                message = `${userName} | ${name}(${(0, _stringify2.default)(params)}) [${(0, _ms2.default)(Date.now() - startTime)}] =!> ${_context2.t1}`;


                this._logger.error(message, data);

                if (this._xo._config.verboseLogsOnErrors) {
                  debug(message);

                  stack = _context2.t1 && _context2.t1.stack;

                  if (stack) {
                    console.error(stack);
                  }
                } else {
                  debug('%s | %s(...) [%s] =!> %s', userName, name, (0, _ms2.default)(Date.now() - startTime), _context2.t1);
                }

                xoError = XAPI_ERROR_TO_XO_ERROR[_context2.t1.code];

                if (!xoError) {
                  _context2.next = 38;
                  break;
                }

                throw xoError(_context2.t1.params, function (ref) {
                  try {
                    return _this4._xo.getObject(ref).id;
                  } catch (e) {
                    return ref;
                  }
                });

              case 38:
                throw _context2.t1;

              case 39:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[13, 29]]);
      }));

      function callApiMethod(_x, _x2, _x3) {
        return _ref22.apply(this, arguments);
      }

      return callApiMethod;
    }()
  }, {
    key: 'apiMethods',
    get: function get() {
      return this._methods;
    }
  }]);
  return Api;
}();

exports.default = Api;
//# sourceMappingURL=api.js.map
