'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.ScheduleAlreadyEnabled = exports.ScheduleNotEnabled = exports.ScheduleOverride = exports.SchedulerError = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _makeError = require('make-error');

var _apiErrors = require('xo-common/api-errors.js');

var _schedule = require('../models/schedule');

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ===================================================================

var _resolveId = function _resolveId(scheduleOrId) {
  return scheduleOrId.id || scheduleOrId;
};

var SchedulerError = exports.SchedulerError = function (_BaseError) {
  (0, _inherits3.default)(SchedulerError, _BaseError);

  function SchedulerError() {
    (0, _classCallCheck3.default)(this, SchedulerError);
    return (0, _possibleConstructorReturn3.default)(this, (SchedulerError.__proto__ || (0, _getPrototypeOf2.default)(SchedulerError)).apply(this, arguments));
  }

  return SchedulerError;
}(_makeError.BaseError);

var ScheduleOverride = exports.ScheduleOverride = function (_SchedulerError) {
  (0, _inherits3.default)(ScheduleOverride, _SchedulerError);

  function ScheduleOverride(scheduleOrId) {
    (0, _classCallCheck3.default)(this, ScheduleOverride);
    return (0, _possibleConstructorReturn3.default)(this, (ScheduleOverride.__proto__ || (0, _getPrototypeOf2.default)(ScheduleOverride)).call(this, 'Schedule ID ' + _resolveId(scheduleOrId) + ' is already added'));
  }

  return ScheduleOverride;
}(SchedulerError);

var ScheduleNotEnabled = exports.ScheduleNotEnabled = function (_SchedulerError2) {
  (0, _inherits3.default)(ScheduleNotEnabled, _SchedulerError2);

  function ScheduleNotEnabled(scheduleOrId) {
    (0, _classCallCheck3.default)(this, ScheduleNotEnabled);
    return (0, _possibleConstructorReturn3.default)(this, (ScheduleNotEnabled.__proto__ || (0, _getPrototypeOf2.default)(ScheduleNotEnabled)).call(this, 'Schedule ' + _resolveId(scheduleOrId) + ' is not enabled'));
  }

  return ScheduleNotEnabled;
}(SchedulerError);

var ScheduleAlreadyEnabled = exports.ScheduleAlreadyEnabled = function (_SchedulerError3) {
  (0, _inherits3.default)(ScheduleAlreadyEnabled, _SchedulerError3);

  function ScheduleAlreadyEnabled(scheduleOrId) {
    (0, _classCallCheck3.default)(this, ScheduleAlreadyEnabled);
    return (0, _possibleConstructorReturn3.default)(this, (ScheduleAlreadyEnabled.__proto__ || (0, _getPrototypeOf2.default)(ScheduleAlreadyEnabled)).call(this, 'Schedule ' + _resolveId(scheduleOrId) + ' is already enabled'));
  }

  return ScheduleAlreadyEnabled;
}(SchedulerError);

// ===================================================================

var _default = function () {
  function _default(xo) {
    var _this5 = this;

    (0, _classCallCheck3.default)(this, _default);

    this.xo = xo;
    var schedules = this._redisSchedules = new _schedule.Schedules({
      connection: xo._redis,
      prefix: 'xo:schedule',
      indexes: ['user_id', 'job']
    });
    this._scheduleTable = undefined;

    xo.on('start', function () {
      xo.addConfigManager('schedules', function () {
        return schedules.get();
      }, function (schedules_) {
        return _promise2.default.all((0, _utils.mapToArray)(schedules_, function (schedule) {
          return schedules.save(schedule);
        }));
      });

      return _this5._loadSchedules();
    });
    xo.on('stop', function () {
      return _this5._disableAll();
    });
  }

  (0, _createClass3.default)(_default, [{
    key: '_add',
    value: function _add(schedule) {
      var id = _resolveId(schedule);
      this._schedules[id] = schedule;
      this._scheduleTable[id] = false;
      if (schedule.enabled) {
        this._enable(schedule);
      }
    }
  }, {
    key: '_exists',
    value: function _exists(scheduleOrId) {
      var id_ = _resolveId(scheduleOrId);
      return id_ in this._schedules;
    }
  }, {
    key: '_isEnabled',
    value: function _isEnabled(scheduleOrId) {
      return this._scheduleTable[_resolveId(scheduleOrId)];
    }
  }, {
    key: '_enable',
    value: function _enable(schedule) {
      var _this6 = this;

      var id = schedule.id;


      var stopSchedule = (0, _utils.scheduleFn)(schedule.cron, function () {
        return _this6.xo.runJobSequence([schedule.job]);
      }, schedule.timezone);

      this._cronJobs[id] = stopSchedule;
      this._scheduleTable[id] = true;
    }
  }, {
    key: '_disable',
    value: function _disable(scheduleOrId) {
      if (!this._exists(scheduleOrId)) {
        throw (0, _apiErrors.noSuchObject)(scheduleOrId, 'schedule');
      }
      if (!this._isEnabled(scheduleOrId)) {
        throw new ScheduleNotEnabled(scheduleOrId);
      }
      var id = _resolveId(scheduleOrId);
      this._cronJobs[id](); // Stop cron job.
      delete this._cronJobs[id];
      this._scheduleTable[id] = false;
    }
  }, {
    key: '_disableAll',
    value: function _disableAll() {
      var _this7 = this;

      (0, _utils.forEach)(this._scheduleTable, function (enabled, id) {
        if (enabled) {
          _this7._disable(id);
        }
      });
    }
  }, {
    key: '_loadSchedules',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var _this8 = this;

        var schedules;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                this._schedules = {};
                this._scheduleTable = {};
                this._cronJobs = {};

                _context.next = 5;
                return this.xo.getAllSchedules();

              case 5:
                schedules = _context.sent;


                (0, _utils.forEach)(schedules, function (schedule) {
                  _this8._add(schedule);
                });

              case 7:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function _loadSchedules() {
        return _ref.apply(this, arguments);
      }

      return _loadSchedules;
    }()
  }, {
    key: '_getSchedule',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(id) {
        var schedule;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this._redisSchedules.first(id);

              case 2:
                schedule = _context2.sent;

                if (schedule) {
                  _context2.next = 5;
                  break;
                }

                throw (0, _apiErrors.noSuchObject)(id, 'schedule');

              case 5:
                return _context2.abrupt('return', schedule);

              case 6:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function _getSchedule(_x) {
        return _ref2.apply(this, arguments);
      }

      return _getSchedule;
    }()
  }, {
    key: 'getSchedule',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(id) {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this._getSchedule(id);

              case 2:
                return _context3.abrupt('return', _context3.sent.properties);

              case 3:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function getSchedule(_x2) {
        return _ref3.apply(this, arguments);
      }

      return getSchedule;
    }()
  }, {
    key: 'getAllSchedules',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                return _context4.abrupt('return', /* await */this._redisSchedules.get());

              case 1:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function getAllSchedules() {
        return _ref4.apply(this, arguments);
      }

      return getAllSchedules;
    }()
  }, {
    key: 'createSchedule',
    value: function () {
      var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5(userId, _ref6) {
        var job = _ref6.job,
            cron = _ref6.cron,
            enabled = _ref6.enabled,
            name = _ref6.name,
            timezone = _ref6.timezone;
        var schedule_, schedule;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return this._redisSchedules.create(userId, job, cron, enabled, name, timezone);

              case 2:
                schedule_ = _context5.sent;
                schedule = schedule_.properties;


                this._add(schedule);

                return _context5.abrupt('return', schedule);

              case 6:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function createSchedule(_x3, _x4) {
        return _ref5.apply(this, arguments);
      }

      return createSchedule;
    }()
  }, {
    key: 'updateSchedule',
    value: function () {
      var _ref7 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6(id, _ref8) {
        var job = _ref8.job,
            cron = _ref8.cron,
            enabled = _ref8.enabled,
            name = _ref8.name,
            timezone = _ref8.timezone;
        var schedule, properties;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return this._getSchedule(id);

              case 2:
                schedule = _context6.sent;


                if (job) schedule.set('job', job);
                if (cron) schedule.set('cron', cron);
                if (enabled !== undefined) schedule.set('enabled', enabled);
                if (name !== undefined) schedule.set('name', name);
                if (timezone === null) {
                  schedule.set('timezone', undefined); // Remove current timezone
                } else if (timezone !== undefined) {
                  schedule.set('timezone', timezone);
                }

                _context6.next = 10;
                return this._redisSchedules.save(schedule);

              case 10:
                properties = schedule.properties;

                if (this._exists(properties)) {
                  _context6.next = 13;
                  break;
                }

                throw (0, _apiErrors.noSuchObject)(properties, 'schedule');

              case 13:
                if (!this._isEnabled(properties)) {
                  _context6.next = 16;
                  break;
                }

                _context6.next = 16;
                return this._disable(properties);

              case 16:

                this._add(properties);

              case 17:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function updateSchedule(_x5, _x6) {
        return _ref7.apply(this, arguments);
      }

      return updateSchedule;
    }()
  }, {
    key: 'removeSchedule',
    value: function () {
      var _ref9 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7(id) {
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _context7.next = 2;
                return this._redisSchedules.remove(id);

              case 2:
                _context7.prev = 2;

                this._disable(id);
                _context7.next = 10;
                break;

              case 6:
                _context7.prev = 6;
                _context7.t0 = _context7['catch'](2);

                if (_context7.t0 instanceof SchedulerError) {
                  _context7.next = 10;
                  break;
                }

                throw _context7.t0;

              case 10:
                _context7.prev = 10;

                delete this._schedules[id];
                delete this._scheduleTable[id];
                return _context7.finish(10);

              case 14:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this, [[2, 6, 10, 14]]);
      }));

      function removeSchedule(_x7) {
        return _ref9.apply(this, arguments);
      }

      return removeSchedule;
    }()
  }, {
    key: 'scheduleTable',
    get: function get() {
      return this._scheduleTable;
    }
  }]);
  return _default;
}();

exports.default = _default;
//# sourceMappingURL=scheduling.js.map
